// Coselica Toolbox for Scicos/Nsp
// Copyright (C) 2009-2016  Dirk Reusch, Kybernetik Dr. Reusch
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.

function [x,y,typ]=CBC_SecondOrder(job,arg1,arg2)

  function blk_draw(sz,orig,orient,label)
    if orient then
      xx=orig(1);yy=orig(2);
      ww=sz(1);hh=sz(2);
    else
      xx=orig(1)+sz(1);yy=orig(2);
      ww=-sz(1);hh=sz(2);
    end
    if orient then
      xrect(orig(1)+sz(1)*0,orig(2)+sz(2)*1,sz(1)*1,sz(2)*1,color=color(0,0,127),background=color(255,255,255),thickness=0.25);
    else
      xrect(orig(1)+sz(1)*(1-0-1),orig(2)+sz(2)*1,sz(1)*1,sz(2)*1,color=color(0,0,127),background=color(255,255,255),thickness=0.25);
    end
    if orient then
      xstring(orig(1)+sz(1)*-0.25,orig(2)+sz(2)*1.05,model.label,w=sz(1)*1.5,h=sz(2)*0.2,fill=%t,color=color(0,0,0));
    else
      xstring(orig(1)+sz(1)*(-0.25),orig(2)+sz(2)*1.05,model.label,w=sz(1)*1.5,h=sz(2)*0.2,fill=%t,color=color(0,0,0));
    end
    xpoly(xx+ww*[0.1;0.1],yy+hh*[0.89;0.05],color=color(192,192,192),thickness=0.25);
    xfpoly(xx+ww*[0.1;0.06;0.14;0.1;0.1],yy+hh*[0.95;0.84;0.84;0.94;0.95],color=color(192,192,192),fill_color=color(192,192,192),thickness=0.25);
    xpoly(xx+ww*[0.05;0.91],yy+hh*[0.1;0.1],color=color(192,192,192),thickness=0.25);
    xfpoly(xx+ww*[0.95;0.84;0.84;0.95],yy+hh*[0.1;0.14;0.06;0.1],color=color(192,192,192),fill_color=color(192,192,192),thickness=0.25);
    xpoly(xx+ww*[0.1;0.14;0.18;0.22;0.26;0.3;0.34;0.38;0.42;0.46;0.5;0.54;0.58;0.62;0.66;0.7;0.74;0.78;0.82;0.86;0.9],yy+hh*[0.1;0.15735;0.3025;0.48739;0.66375;0.794;0.85755;0.85245;0.79225;0.7003;0.60275;0.522295;0.473645;0.461855;0.48286;0.52605;0.5778;0.62515;0.6583;0.6725;0.66805],color=color(0,0,0),thickness=0.25);
    if orient then
      xstring(orig(1)+sz(1)*0.5,orig(2)+sz(2)*0.15,"PT2",w=sz(1)*0.3,h=sz(2)*0.3,fill=%t,color=color(0,0,0));
    else
      xstring(orig(1)+sz(1)*(0.2),orig(2)+sz(2)*0.15,"PT2",w=sz(1)*0.3,h=sz(2)*0.3,fill=%t,color=color(0,0,0));
    end
    if orient then
      xstring(orig(1)+sz(1)*-0.25,orig(2)+sz(2)*-0.25,"w="+string(w)+"",w=sz(1)*1.5,h=sz(2)*0.2,fill=%t,color=color(0,0,0));
    else
      xstring(orig(1)+sz(1)*(-0.25),orig(2)+sz(2)*-0.25,"w="+string(w)+"",w=sz(1)*1.5,h=sz(2)*0.2,fill=%t,color=color(0,0,0));
    end
  endfunction

x=[];y=[];typ=[];

select job
  case 'plot' then
    k=arg1.graphics.exprs(1);
    w=arg1.graphics.exprs(2);
    D=arg1.graphics.exprs(3);
    standard_draw(arg1,%f,_MBI_SISO_dp);
  case 'getinputs' then
    [x,y,typ]=_MBI_SISO_ip(arg1);
  case 'getoutputs' then
    [x,y,typ]=_MBI_SISO_op(arg1);
  case 'getorigin' then
    [x,y]=standard_origin(arg1);
  case 'set' then
    x=arg1;
    graphics=arg1.graphics;exprs=graphics.exprs;
    model=arg1.model;
    while %t do
      [ok,k,w,D,exprs]=..
        getvalue(['';'CBC_SecondOrder';'';'Second order transfer function block (= 2 poles)';''],..
        [' k [-] : Gain',' w [-] : Natural angular frequency of undamped system (w>0)',' D [-] : Damping'],..
        list('vec',1,'vec',1,'vec',1),exprs);
      if ~ok then break, end
      model.equations.parameters(2)=list(k,w,D)
      model.in=[1];
      model.out=[1];
      graphics.exprs=exprs;
      x.graphics=graphics;x.model=model;
      break
    end
  case 'define' then
    k=1;
    w=1;
    D=1;
    model=scicos_model();
    model.sim='Coselica';
    model.blocktype='c';
    model.dep_ut=[%t %f];
    model.in=[1];
    model.out=[1];
    mo=modelica();
      mo.model='Coselica.Blocks.Continuous.SecondOrder';
      mo.inputs=['u'];
      mo.outputs=['y'];
      mo.parameters=list(['k','w','D'],..
                         list(k,w,D),..
                         [0,0,0]);
    model.equations=mo;
    exprs=[strcat(sci2exp(k));strcat(sci2exp(w));strcat(sci2exp(D))];
    gr_i="blk_draw(sz,orig,orient,model.label)";
    x=standard_define([2 2],model,exprs,list(gr_i,0));
    x.graphics.in_implicit=['E'];
    x.graphics.out_implicit=['E'];
  end
endfunction
