// Coselica Toolbox for Scicos/Nsp
// Copyright (C) 2009-2016  Dirk Reusch, Kybernetik Dr. Reusch
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.

function [x,y,typ]=CBMV_MatrixGain(job,arg1,arg2)

  function blk_draw(sz,orig,orient,label)
    if orient then
      xx=orig(1);yy=orig(2);
      ww=sz(1);hh=sz(2);
    else
      xx=orig(1)+sz(1);yy=orig(2);
      ww=-sz(1);hh=sz(2);
    end
    if orient then
      xrect(orig(1)+sz(1)*0,orig(2)+sz(2)*1,sz(1)*1,sz(2)*1,color=color(0,0,127),background=color(255,255,255),thickness=0.25);
    else
      xrect(orig(1)+sz(1)*(1-0-1),orig(2)+sz(2)*1,sz(1)*1,sz(2)*1,color=color(0,0,127),background=color(255,255,255),thickness=0.25);
    end
    if orient then
      xstring(orig(1)+sz(1)*-0.25,orig(2)+sz(2)*1.05,model.label,w=sz(1)*1.5,h=sz(2)*0.2,fill=%t,color=color(0,0,0));
    else
      xstring(orig(1)+sz(1)*(-0.25),orig(2)+sz(2)*1.05,model.label,w=sz(1)*1.5,h=sz(2)*0.2,fill=%t,color=color(0,0,0));
    end
    if orient then
      xstring(orig(1)+sz(1)*0.05,orig(2)+sz(2)*0.2,"*K",w=sz(1)*0.9,h=sz(2)*0.6,fill=%t,color=color(160,160,164));
    else
      xstring(orig(1)+sz(1)*(0.05),orig(2)+sz(2)*0.2,"*K",w=sz(1)*0.9,h=sz(2)*0.6,fill=%t,color=color(160,160,164));
    end
    if orient then
      xstring(orig(1)+sz(1)*-0.25,orig(2)+sz(2)*1.05,model.label,w=sz(1)*1.5,h=sz(2)*0.2,fill=%t,color=color(0,0,255));
    else
      xstring(orig(1)+sz(1)*(-0.25),orig(2)+sz(2)*1.05,model.label,w=sz(1)*1.5,h=sz(2)*0.2,fill=%t,color=color(0,0,255));
    end
  endfunction

x=[];y=[];typ=[];

select job
  case 'plot' then
    nin=arg1.graphics.exprs(1);
    nout=arg1.graphics.exprs(2);
    K=arg1.graphics.exprs(3);
    nin=strcat(sci2exp(double(evstr(nin))));
    nout=strcat(sci2exp(double(evstr(nout))));
    standard_draw(arg1,%f,_MBI_SISO_dp);
  case 'getinputs' then
    [x,y,typ]=_MBI_SISO_ip(arg1);
  case 'getoutputs' then
    [x,y,typ]=_MBI_SISO_op(arg1);
  case 'getorigin' then
    [x,y]=standard_origin(arg1);
  case 'set' then
    x=arg1;
    graphics=arg1.graphics;exprs=graphics.exprs;
    model=arg1.model;
    while %t do
      [ok,nin,nout,K,exprs]=..
        getvalue(['';'CBMV_MatrixGain';'';'Output the product of a gain matrix with the input signal vector';''],..
        [' nin [-] : Number of inputs (>1)',' nout [-] : Number of outputs (>1)',' K [-] : Gain matrix (nout rows, nin columns)'],..
        list('vec',1,'vec',1,'mat',[-1,-1]),exprs);
      if ~ok then break, end
      model.equations.parameters(2)=list(nin,nout,K)
      nin=double(nin);
      nout=double(nout);
      model.in=[nin];
      model.out=[nout];
      graphics.exprs=exprs;
      x.graphics=graphics;x.model=model;
      break
    end
  case 'define' then
    nin=2;
    nout=2;
    K=[1,0;0,1];
    model=scicos_model();
    model.sim='Coselica';
    model.blocktype='c';
    model.dep_ut=[%t %f];
    model.in=[nin];
    model.out=[nout];
    nin=int32(2);
    nout=int32(2);
    mo=modelica();
      mo.model='Coselica.Blocks.Math.Vectors.MatrixGain';
      mo.inputs=['u'];
      mo.outputs=['y'];
      mo.parameters=list(['nin','nout','K'],..
                         list(nin,nout,K),..
                         [0,0,0]);
    model.equations=mo;
    exprs=[strcat(sci2exp(nin));strcat(sci2exp(nout));strcat(sci2exp(K))];
    gr_i="blk_draw(sz,orig,orient,model.label)";
    x=standard_define([2 2],model,exprs,list(gr_i,0));
    x.graphics.in_implicit=['E'];
    x.graphics.out_implicit=['E'];
  end
endfunction
