// Coselica Toolbox for Scicos/Nsp
// Copyright (C) 2009-2016  Dirk Reusch, Kybernetik Dr. Reusch
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.

function [x,y,typ]=CBN_RateLimiter(job,arg1,arg2)

  function blk_draw(sz,orig,orient,label)
    if orient then
      xx=orig(1);yy=orig(2);
      ww=sz(1);hh=sz(2);
    else
      xx=orig(1)+sz(1);yy=orig(2);
      ww=-sz(1);hh=sz(2);
    end
    if orient then
      xrect(orig(1)+sz(1)*0,orig(2)+sz(2)*1,sz(1)*1,sz(2)*1,color=color(0,0,127),background=color(255,255,255),thickness=0.25);
    else
      xrect(orig(1)+sz(1)*(1-0-1),orig(2)+sz(2)*1,sz(1)*1,sz(2)*1,color=color(0,0,127),background=color(255,255,255),thickness=0.25);
    end
    if orient then
      xstring(orig(1)+sz(1)*-0.25,orig(2)+sz(2)*1.05,model.label,w=sz(1)*1.5,h=sz(2)*0.2,fill=%t,color=color(0,0,0));
    else
      xstring(orig(1)+sz(1)*(-0.25),orig(2)+sz(2)*1.05,model.label,w=sz(1)*1.5,h=sz(2)*0.2,fill=%t,color=color(0,0,0));
    end
    xpoly(xx+ww*[0.5;0.5],yy+hh*[0.05;0.84],color=color(192,192,192),thickness=0.25);
    xfpoly(xx+ww*[0.5;0.54;0.54;0.5],yy+hh*[0.95;0.84;0.84;0.95],color=color(192,192,192),fill_color=color(192,192,192),thickness=0.25);
    xpoly(xx+ww*[0.05;0.84],yy+hh*[0.5;0.5],color=color(192,192,192),thickness=0.25);
    xfpoly(xx+ww*[0.95;0.84;0.84;0.95],yy+hh*[0.5;0.46;0.54;0.5],color=color(192,192,192),fill_color=color(192,192,192),thickness=0.25);
    xpoly(xx+ww*[0.25;0.75],yy+hh*[0.15;0.85],color=color(0,0,0),thickness=0.25);
    if orient then
      xstring(orig(1)+sz(1)*-0.25,orig(2)+sz(2)*-0.25,"sMax="+string(sMax)+"",w=sz(1)*1.5,h=sz(2)*0.2,fill=%t,color=color(0,0,0));
    else
      xstring(orig(1)+sz(1)*(-0.25),orig(2)+sz(2)*-0.25,"sMax="+string(sMax)+"",w=sz(1)*1.5,h=sz(2)*0.2,fill=%t,color=color(0,0,0));
    end
    if orient then
      xstring(orig(1)+sz(1)*-0.25,orig(2)+sz(2)*1.05,model.label,w=sz(1)*1.5,h=sz(2)*0.2,fill=%t,color=color(0,0,0));
    else
      xstring(orig(1)+sz(1)*(-0.25),orig(2)+sz(2)*1.05,model.label,w=sz(1)*1.5,h=sz(2)*0.2,fill=%t,color=color(0,0,0));
    end
  endfunction

x=[];y=[];typ=[];

select job
  case 'plot' then
    sMax=arg1.graphics.exprs(1);
    sMin=arg1.graphics.exprs(2);
    T=arg1.graphics.exprs(3);
    y_start=arg1.graphics.exprs(4);
    standard_draw(arg1,%f,_MBI_SISO_dp);
  case 'getinputs' then
    [x,y,typ]=_MBI_SISO_ip(arg1);
  case 'getoutputs' then
    [x,y,typ]=_MBI_SISO_op(arg1);
  case 'getorigin' then
    [x,y]=standard_origin(arg1);
  case 'set' then
    x=arg1;
    graphics=arg1.graphics;exprs=graphics.exprs;
    model=arg1.model;
    while %t do
      [ok,sMax,sMin,T,y_start,exprs]=..
        getvalue(['';'CBN_RateLimiter';'';'Limit rate of change of signal';''],..
        [' sMax [-] : Maximum slope',' sMin [-] : Minimum slope',' T [s] : Time constant (> 0)',' y_start [-] : Start value of output'],..
        list('vec',1,'vec',1,'vec',1,'vec',1),exprs);
      if ~ok then break, end
      model.equations.parameters(2)=list(sMax,sMin,T,y_start)
      model.in=[1];
      model.out=[1];
      graphics.exprs=exprs;
      x.graphics=graphics;x.model=model;
      break
    end
  case 'define' then
    sMax=1;
    sMin=-1;
    T=0.0001;
    y_start=0;
    model=scicos_model();
    model.sim='Coselica';
    model.blocktype='c';
    model.dep_ut=[%t %f];
    model.in=[1];
    model.out=[1];
    mo=modelica();
      mo.model='Coselica.Blocks.Nonlinear.RateLimiter';
      mo.inputs=['u'];
      mo.outputs=['y'];
      mo.parameters=list(['sMax','sMin','T','y_start'],..
                         list(sMax,sMin,T,y_start),..
                         [0,0,0,0]);
    model.equations=mo;
    exprs=[strcat(sci2exp(sMax));strcat(sci2exp(sMin));strcat(sci2exp(T));strcat(sci2exp(y_start))];
    gr_i="blk_draw(sz,orig,orient,model.label)";
    x=standard_define([2 2],model,exprs,list(gr_i,0));
    x.graphics.in_implicit=['E'];
    x.graphics.out_implicit=['E'];
  end
endfunction
