// Coselica Toolbox for Scicos/Nsp
// Copyright (C) 2009-2016  Dirk Reusch, Kybernetik Dr. Reusch
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.

function [x,y,typ]=CBS_ExpSine(job,arg1,arg2)

  function blk_draw(sz,orig,orient,label)
    if orient then
      xx=orig(1);yy=orig(2);
      ww=sz(1);hh=sz(2);
    else
      xx=orig(1)+sz(1);yy=orig(2);
      ww=-sz(1);hh=sz(2);
    end
    if orient then
      xrect(orig(1)+sz(1)*0,orig(2)+sz(2)*1,sz(1)*1,sz(2)*1,color=color(0,0,127),background=color(255,255,255),thickness=0.25);
    else
      xrect(orig(1)+sz(1)*(1-0-1),orig(2)+sz(2)*1,sz(1)*1,sz(2)*1,color=color(0,0,127),background=color(255,255,255),thickness=0.25);
    end
    if orient then
      xstring(orig(1)+sz(1)*-0.25,orig(2)+sz(2)*1.05,model.label,w=sz(1)*1.5,h=sz(2)*0.2,fill=%t,color=color(0,0,0));
    else
      xstring(orig(1)+sz(1)*(-0.25),orig(2)+sz(2)*1.05,model.label,w=sz(1)*1.5,h=sz(2)*0.2,fill=%t,color=color(0,0,0));
    end
    xpoly(xx+ww*[0.1;0.1],yy+hh*[0.84;0.1],color=color(192,192,192),thickness=0.25);
    xfpoly(xx+ww*[0.1;0.06;0.14;0.1],yy+hh*[0.95;0.84;0.84;0.95],color=color(192,192,192),fill_color=color(192,192,192),thickness=0.25);
    xpoly(xx+ww*[0.05;0.84],yy+hh*[0.5;0.5],color=color(192,192,192),thickness=0.25);
    xfpoly(xx+ww*[0.95;0.84;0.84;0.95],yy+hh*[0.5;0.54;0.46;0.5],color=color(192,192,192),fill_color=color(192,192,192),thickness=0.25);
    xpoly(xx+ww*[0.1;0.124;0.14;0.1565;0.1725;0.1885;0.2045;0.2205;0.2365;0.257;0.285;0.325;0.349;0.3695;0.3895;0.4095;0.4295;0.4495;0.47385;0.5422;0.5665;0.5905;0.6105;0.6345;0.659;0.691;0.7555;0.7875;0.8155;0.8435;0.876;0.9],yy+hh*[0.5;0.6615;0.7515;0.8225;0.871;0.8965;0.898;0.8765;0.8355;0.761;0.629;0.4305;0.3315;0.2705;0.234;0.2235;0.2375;0.2735;0.3395;0.5685;0.632;0.674;0.69;0.686;0.659;0.597;0.4475;0.394;0.3705;0.3705;0.3975;0.431],color=color(0,0,0),thickness=0.25);
    if orient then
      xstring(orig(1)+sz(1)*-0.235,orig(2)+sz(2)*-0.26,"freqHz="+string(freqHz)+"",w=sz(1)*1.5,h=sz(2)*0.2,fill=%t,color=color(0,0,0));
    else
      xstring(orig(1)+sz(1)*(-0.265),orig(2)+sz(2)*-0.26,"freqHz="+string(freqHz)+"",w=sz(1)*1.5,h=sz(2)*0.2,fill=%t,color=color(0,0,0));
    end
  endfunction

x=[];y=[];typ=[];

select job
  case 'plot' then
    amplitude=arg1.graphics.exprs(1);
    freqHz=arg1.graphics.exprs(2);
    phase=arg1.graphics.exprs(3);
    damping=arg1.graphics.exprs(4);
    offset=arg1.graphics.exprs(5);
    startTime=arg1.graphics.exprs(6);
    standard_draw(arg1,%f,_MBI_SO_dp);
  case 'getinputs' then
    [x,y,typ]=_MBI_SO_ip(arg1);
  case 'getoutputs' then
    [x,y,typ]=_MBI_SO_op(arg1);
  case 'getorigin' then
    [x,y]=standard_origin(arg1);
  case 'set' then
    x=arg1;
    graphics=arg1.graphics;exprs=graphics.exprs;
    model=arg1.model;
    while %t do
      [ok,amplitude,freqHz,phase,damping,offset,startTime,exprs]=..
        getvalue(['';'CBS_ExpSine';'';'Generate exponentially damped sine signal';''],..
        [' amplitude [-] : Amplitude of sine wave',' freqHz [Hz] : Frequency of sine wave',' phase [rad] : Phase of sine wave',' damping [s-1] : Damping coefficient of sine wave',' offset [-] : Offset of output signal',' startTime [s] : Output = offset for time < startTime'],..
        list('vec',1,'vec',1,'vec',1,'vec',1,'vec',1,'vec',1),exprs);
      if ~ok then break, end
      model.equations.parameters(2)=list(amplitude,freqHz,phase,damping,offset,startTime)
      model.in=[];
      model.out=[1];
      graphics.exprs=exprs;
      x.graphics=graphics;x.model=model;
      break
    end
  case 'define' then
    amplitude=1;
    freqHz=2;
    phase=0;
    damping=1;
    offset=0;
    startTime=0;
    model=scicos_model();
    model.sim='Coselica';
    model.blocktype='c';
    model.dep_ut=[%t %f];
    model.in=[];
    model.out=[1];
    mo=modelica();
      mo.model='Coselica.Blocks.Sources.ExpSine';
      mo.inputs=[];
      mo.outputs=['y'];
      mo.parameters=list(['amplitude','freqHz','phase','damping','offset','startTime'],..
                         list(amplitude,freqHz,phase,damping,offset,startTime),..
                         [0,0,0,0,0,0]);
    model.equations=mo;
    exprs=[strcat(sci2exp(amplitude));strcat(sci2exp(freqHz));strcat(sci2exp(phase));strcat(sci2exp(damping));strcat(sci2exp(offset));strcat(sci2exp(startTime))];
    gr_i="blk_draw(sz,orig,orient,model.label)";
    x=standard_define([2 2],model,exprs,list(gr_i,0));
    x.graphics.in_implicit=[];
    x.graphics.out_implicit=['E'];
  end
endfunction
