// Coselica Toolbox for Scicos/Nsp
// Copyright (C) 2009-2016  Dirk Reusch, Kybernetik Dr. Reusch
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.

function [x,y,typ]=CBS_Ramp(job,arg1,arg2)

  function blk_draw(sz,orig,orient,label)
    if orient then
      xx=orig(1);yy=orig(2);
      ww=sz(1);hh=sz(2);
    else
      xx=orig(1)+sz(1);yy=orig(2);
      ww=-sz(1);hh=sz(2);
    end
    if orient then
      xrect(orig(1)+sz(1)*0,orig(2)+sz(2)*1,sz(1)*1,sz(2)*1,color=color(0,0,127),background=color(255,255,255),thickness=0.25);
    else
      xrect(orig(1)+sz(1)*(1-0-1),orig(2)+sz(2)*1,sz(1)*1,sz(2)*1,color=color(0,0,127),background=color(255,255,255),thickness=0.25);
    end
    if orient then
      xstring(orig(1)+sz(1)*-0.25,orig(2)+sz(2)*1.05,model.label,w=sz(1)*1.5,h=sz(2)*0.2,fill=%t,color=color(0,0,0));
    else
      xstring(orig(1)+sz(1)*(-0.25),orig(2)+sz(2)*1.05,model.label,w=sz(1)*1.5,h=sz(2)*0.2,fill=%t,color=color(0,0,0));
    end
    xpoly(xx+ww*[0.1;0.1],yy+hh*[0.84;0.1],color=color(192,192,192),thickness=0.25);
    xfpoly(xx+ww*[0.1;0.06;0.14;0.1],yy+hh*[0.95;0.84;0.84;0.95],color=color(192,192,192),fill_color=color(192,192,192),thickness=0.25);
    xpoly(xx+ww*[0.05;0.91],yy+hh*[0.15;0.15],color=color(192,192,192),thickness=0.25);
    xfpoly(xx+ww*[0.95;0.84;0.84;0.95],yy+hh*[0.15;0.19;0.11;0.15],color=color(192,192,192),fill_color=color(192,192,192),thickness=0.25);
    xpoly(xx+ww*[0.1;0.3;0.655],yy+hh*[0.15;0.15;0.69],color=color(0,0,0),thickness=0.25);
    if orient then
      xstring(orig(1)+sz(1)*-0.25,orig(2)+sz(2)*-0.25,"duration="+string(duration)+"",w=sz(1)*1.5,h=sz(2)*0.2,fill=%t,color=color(0,0,0));
    else
      xstring(orig(1)+sz(1)*(-0.25),orig(2)+sz(2)*-0.25,"duration="+string(duration)+"",w=sz(1)*1.5,h=sz(2)*0.2,fill=%t,color=color(0,0,0));
    end
    xpoly(xx+ww*[0.655;0.93],yy+hh*[0.69;0.69],color=color(0,0,0),thickness=0.25);
  endfunction

x=[];y=[];typ=[];

select job
  case 'plot' then
    height=arg1.graphics.exprs(1);
    duration=arg1.graphics.exprs(2);
    offset=arg1.graphics.exprs(3);
    startTime=arg1.graphics.exprs(4);
    standard_draw(arg1,%f,_MBI_SO_dp);
  case 'getinputs' then
    [x,y,typ]=_MBI_SO_ip(arg1);
  case 'getoutputs' then
    [x,y,typ]=_MBI_SO_op(arg1);
  case 'getorigin' then
    [x,y]=standard_origin(arg1);
  case 'set' then
    x=arg1;
    graphics=arg1.graphics;exprs=graphics.exprs;
    model=arg1.model;
    while %t do
      [ok,height,duration,offset,startTime,exprs]=..
        getvalue(['';'CBS_Ramp';'';'Generate ramp signal';''],..
        [' height [-] : Height of ramps',' duration [-] : Durations of ramp',' offset [-] : Offset of output signal',' startTime [s] : Output = offset for time < startTime'],..
        list('vec',1,'vec',1,'vec',1,'vec',1),exprs);
      if ~ok then break, end
      model.equations.parameters(2)=list(height,duration,offset,startTime)
      model.in=[];
      model.out=[1];
      graphics.exprs=exprs;
      x.graphics=graphics;x.model=model;
      break
    end
  case 'define' then
    height=1;
    duration=2;
    offset=0;
    startTime=0;
    model=scicos_model();
    model.sim='Coselica';
    model.blocktype='c';
    model.dep_ut=[%t %f];
    model.in=[];
    model.out=[1];
    mo=modelica();
      mo.model='Coselica.Blocks.Sources.Ramp';
      mo.inputs=[];
      mo.outputs=['y'];
      mo.parameters=list(['height','duration','offset','startTime'],..
                         list(height,duration,offset,startTime),..
                         [0,0,0,0]);
    model.equations=mo;
    exprs=[strcat(sci2exp(height));strcat(sci2exp(duration));strcat(sci2exp(offset));strcat(sci2exp(startTime))];
    gr_i="blk_draw(sz,orig,orient,model.label)";
    x=standard_define([2 2],model,exprs,list(gr_i,0));
    x.graphics.in_implicit=[];
    x.graphics.out_implicit=['E'];
  end
endfunction
