// Coselica Toolbox for Scicos/Nsp
// Copyright (C) 2009-2016  Dirk Reusch, Kybernetik Dr. Reusch
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.

function [x,y,typ]=CBS_Trapezoid(job,arg1,arg2)

  function blk_draw(sz,orig,orient,label)
    if orient then
      xx=orig(1);yy=orig(2);
      ww=sz(1);hh=sz(2);
    else
      xx=orig(1)+sz(1);yy=orig(2);
      ww=-sz(1);hh=sz(2);
    end
    if orient then
      xrect(orig(1)+sz(1)*0,orig(2)+sz(2)*1,sz(1)*1,sz(2)*1,color=color(0,0,127),background=color(255,255,255),thickness=0.25);
    else
      xrect(orig(1)+sz(1)*(1-0-1),orig(2)+sz(2)*1,sz(1)*1,sz(2)*1,color=color(0,0,127),background=color(255,255,255),thickness=0.25);
    end
    if orient then
      xstring(orig(1)+sz(1)*-0.25,orig(2)+sz(2)*1.05,model.label,w=sz(1)*1.5,h=sz(2)*0.2,fill=%t,color=color(0,0,0));
    else
      xstring(orig(1)+sz(1)*(-0.25),orig(2)+sz(2)*1.05,model.label,w=sz(1)*1.5,h=sz(2)*0.2,fill=%t,color=color(0,0,0));
    end
    xpoly(xx+ww*[0.1;0.1],yy+hh*[0.84;0.1],color=color(192,192,192),thickness=0.25);
    xfpoly(xx+ww*[0.1;0.06;0.14;0.1],yy+hh*[0.95;0.84;0.84;0.95],color=color(192,192,192),fill_color=color(192,192,192),thickness=0.25);
    xpoly(xx+ww*[0.05;0.91],yy+hh*[0.15;0.15],color=color(192,192,192),thickness=0.25);
    xfpoly(xx+ww*[0.95;0.84;0.84;0.95],yy+hh*[0.15;0.19;0.11;0.15],color=color(192,192,192),fill_color=color(192,192,192),thickness=0.25);
    if orient then
      xstring(orig(1)+sz(1)*-0.235,orig(2)+sz(2)*-0.26,"period="+string(period)+"",w=sz(1)*1.5,h=sz(2)*0.2,fill=%t,color=color(0,0,0));
    else
      xstring(orig(1)+sz(1)*(-0.265),orig(2)+sz(2)*-0.26,"period="+string(period)+"",w=sz(1)*1.5,h=sz(2)*0.2,fill=%t,color=color(0,0,0));
    end
    xpoly(xx+ww*[0.095;0.2;0.35;0.545;0.695;0.805;0.95],yy+hh*[0.15;0.15;0.7;0.7;0.15;0.15;0.7],color=color(0,0,0),thickness=0.25);
  endfunction

x=[];y=[];typ=[];

select job
  case 'plot' then
    amplitude=arg1.graphics.exprs(1);
    rising=arg1.graphics.exprs(2);
    width=arg1.graphics.exprs(3);
    falling=arg1.graphics.exprs(4);
    period=arg1.graphics.exprs(5);
    nperiod=arg1.graphics.exprs(6);
    offset=arg1.graphics.exprs(7);
    startTime=arg1.graphics.exprs(8);
    standard_draw(arg1,%f,_MBI_SO_dp);
  case 'getinputs' then
    [x,y,typ]=_MBI_SO_ip(arg1);
  case 'getoutputs' then
    [x,y,typ]=_MBI_SO_op(arg1);
  case 'getorigin' then
    [x,y]=standard_origin(arg1);
  case 'set' then
    x=arg1;
    graphics=arg1.graphics;exprs=graphics.exprs;
    model=arg1.model;
    while %t do
      [ok,amplitude,rising,width,falling,period,nperiod,offset,startTime,exprs]=..
        getvalue(['';'CBS_Trapezoid';'';'Generate trapezoidal signal';''],..
        [' amplitude [-] : Amplitude of trapezoid',' rising [s] : Rising duration of trapezoid',' width [s] : Width duration of trapezoid',' falling [s] : Falling duration of trapezoid',' period [s] : Time for one period',' nperiod [-] : Number of periods (< 0 means infinite number of periods)',' offset [-] : Offset of output signal',' startTime [s] : Output = offset for time < startTime'],..
        list('vec',1,'vec',1,'vec',1,'vec',1,'vec',1,'vec',1,'vec',1,'vec',1),exprs);
      if ~ok then break, end
      model.equations.parameters(2)=list(amplitude,rising,width,falling,period,nperiod,offset,startTime)
      model.in=[];
      model.out=[1];
      graphics.exprs=exprs;
      x.graphics=graphics;x.model=model;
      break
    end
  case 'define' then
    amplitude=1;
    rising=0;
    width=0.5;
    falling=0;
    period=1;
    nperiod=-1;
    offset=0;
    startTime=0;
    model=scicos_model();
    model.sim='Coselica';
    model.blocktype='c';
    model.dep_ut=[%t %f];
    model.in=[];
    model.out=[1];
    mo=modelica();
      mo.model='Coselica.Blocks.Sources.Trapezoid';
      mo.inputs=[];
      mo.outputs=['y'];
      mo.parameters=list(['amplitude','rising','width','falling','period','nperiod','offset','startTime'],..
                         list(amplitude,rising,width,falling,period,nperiod,offset,startTime),..
                         [0,0,0,0,0,0,0,0]);
    model.equations=mo;
    exprs=[strcat(sci2exp(amplitude));strcat(sci2exp(rising));strcat(sci2exp(width));strcat(sci2exp(falling));strcat(sci2exp(period));strcat(sci2exp(nperiod));strcat(sci2exp(offset));strcat(sci2exp(startTime))];
    gr_i="blk_draw(sz,orig,orient,model.label)";
    x=standard_define([2 2],model,exprs,list(gr_i,0));
    x.graphics.in_implicit=[];
    x.graphics.out_implicit=['E'];
  end
endfunction
