// Coselica Toolbox for Scicos/Nsp
// Copyright (C) 2009-2016  Dirk Reusch, Kybernetik Dr. Reusch
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.

function [x,y,typ]=CEAB_EMF0(job,arg1,arg2)

  function blk_draw(sz,orig,orient,label)
    if orient then
      xx=orig(1);yy=orig(2);
      ww=sz(1);hh=sz(2);
    else
      xx=orig(1)+sz(1);yy=orig(2);
      ww=-sz(1);hh=sz(2);
    end
    if orient then
      xrect(orig(1)+sz(1)*0.075,orig(2)+sz(2)*0.55,sz(1)*0.245,sz(2)*0.1,color=color(0,0,0),background=color(192,192,192),thickness=0.25);
    else
      xrect(orig(1)+sz(1)*(1-0.075-0.245),orig(2)+sz(2)*0.55,sz(1)*0.245,sz(2)*0.1,color=color(0,0,0),background=color(192,192,192),thickness=0.25);
    end
    xpoly(xx+ww*[0.5;0.5],yy+hh*[0.95;0.7],color=color(0,0,255),thickness=0.25);
    if orient then
      xrect(orig(1)+sz(1)*0.675,orig(2)+sz(2)*0.55,sz(1)*0.325,sz(2)*0.1,color=color(0,0,0),background=color(192,192,192),thickness=0.25);
    else
      xrect(orig(1)+sz(1)*(1-0.675-0.325),orig(2)+sz(2)*0.55,sz(1)*0.325,sz(2)*0.1,color=color(0,0,0),background=color(192,192,192),thickness=0.25);
    end
    if orient then
      xarc(orig(1)+sz(1)*0.3,orig(2)+sz(2)*0.7,sz(1)*0.4,sz(2)*0.4,0,23040,color=color(0,0,255),background=color(255,255,255),thickness=0.25);
    else
      xarc(orig(1)+sz(1)*(1-0.3-0.4),orig(2)+sz(2)*0.7,sz(1)*0.4,sz(2)*0.4,-11520,23040,color=color(0,0,255),background=color(255,255,255),thickness=0.25);
    end
    xpoly(xx+ww*[0.5;0.5],yy+hh*[0.05;0.3],color=color(0,0,255),thickness=0.25);
    if orient then
      xstring(orig(1)+sz(1)*0.5,orig(2)+sz(2)*0.05,model.label,w=sz(1)*0.995,h=sz(2)*0.2,fill=%t,color=color(0,0,255));
    else
      xstring(orig(1)+sz(1)*(-0.495),orig(2)+sz(2)*0.05,model.label,w=sz(1)*0.995,h=sz(2)*0.2,fill=%t,color=color(0,0,255));
    end
    if orient then
      xstring(orig(1)+sz(1)*0.5,orig(2)+sz(2)*0.73,"k="+string(k)+"",w=sz(1)*0.945,h=sz(2)*0.17,fill=%t,color=color(0,0,0));
    else
      xstring(orig(1)+sz(1)*(-0.445),orig(2)+sz(2)*0.73,"k="+string(k)+"",w=sz(1)*0.945,h=sz(2)*0.17,fill=%t,color=color(0,0,0));
    end
    xpoly(xx+ww*[0;0.3],yy+hh*[0.35;0.35],color=color(0,0,0),thickness=0.25);
    xpoly(xx+ww*[0;0.1],yy+hh*[0.25;0.35],color=color(0,0,0),thickness=0.25);
    xpoly(xx+ww*[0.1;0.2],yy+hh*[0.25;0.35],color=color(0,0,0),thickness=0.25);
    xpoly(xx+ww*[0.2;0.3],yy+hh*[0.25;0.35],color=color(0,0,0),thickness=0.25);
    xpoly(xx+ww*[0.15;0.15],yy+hh*[0.35;0.45],color=color(0,0,0),thickness=0.25);
  endfunction

x=[];y=[];typ=[];

select job
  case 'plot' then
    k=arg1.graphics.exprs(1);
    standard_draw(arg1,%f,_CEAB_EMF0_dp);
  case 'getinputs' then
    [x,y,typ]=_CEAB_EMF0_ip(arg1);
  case 'getoutputs' then
    [x,y,typ]=_CEAB_EMF0_op(arg1);
  case 'getorigin' then
    [x,y]=standard_origin(arg1);
  case 'set' then
    x=arg1;
    graphics=arg1.graphics;exprs=graphics.exprs;
    model=arg1.model;
    while %t do
      [ok,k,exprs]=..
        getvalue(['';'CEAB_EMF0';'';'Electromotoric force (electric/mechanic transformer)';''],..
        [' k [N.m/A] : Transformation coefficient'],..
        list('vec',1),exprs);
      if ~ok then break, end
      model.equations.parameters(2)=list(k)
      model.in=[1];
      model.out=[1;1];
      graphics.exprs=exprs;
      x.graphics=graphics;x.model=model;
      break
    end
  case 'define' then
    k=1;
    model=scicos_model();
    model.sim='Coselica';
    model.blocktype='c';
    model.dep_ut=[%t %f];
    model.in=[1];
    model.out=[1;1];
    mo=modelica();
      mo.model='Coselica.Electrical.Analog.Basic.EMF0';
      mo.inputs=['p'];
      mo.outputs=['n','flange'];
      mo.parameters=list(['k'],..
                         list(k),..
                         [0]);
    model.equations=mo;
    exprs=[strcat(sci2exp(k))];
    gr_i="blk_draw(sz,orig,orient,model.label)";
    x=standard_define([2 2],model,exprs,list(gr_i,0));
    x.graphics.in_implicit=['I'];
    x.graphics.out_implicit=['I','I'];
  end
endfunction
