// Coselica Toolbox for Scicos/Nsp
// Copyright (C) 2009-2016  Dirk Reusch, Kybernetik Dr. Reusch
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.

function [x,y,typ]=CEAS_PMOS(job,arg1,arg2)

  function blk_draw(sz,orig,orient,label)
    if orient then
      xx=orig(1);yy=orig(2);
      ww=sz(1);hh=sz(2);
    else
      xx=orig(1)+sz(1);yy=orig(2);
      ww=-sz(1);hh=sz(2);
    end
    if orient then
      xstring(orig(1)+sz(1)*0,orig(2)+sz(2)*0.8,model.label,w=sz(1)*1,h=sz(2)*0.2,fill=%t,color=color(0,0,255));
    else
      xstring(orig(1)+sz(1)*(0),orig(2)+sz(2)*0.8,model.label,w=sz(1)*1,h=sz(2)*0.2,fill=%t,color=color(0,0,255));
    end
    xpoly(xx+ww*[0.05;0.45],yy+hh*[0.25;0.25],color=color(0,0,255),thickness=0.25);
    xpoly(xx+ww*[0.45;0.45],yy+hh*[0.25;0.75],color=color(0,0,255),thickness=0.25);
    xpoly(xx+ww*[0.55;0.55],yy+hh*[0.85;0.645],color=color(0,0,255),thickness=0.25);
    xpoly(xx+ww*[0.55;0.55],yy+hh*[0.6;0.395],color=color(0,0,255),thickness=0.25);
    xpoly(xx+ww*[0.55;0.55],yy+hh*[0.35;0.145],color=color(0,0,255),thickness=0.25);
    xpoly(xx+ww*[0.55;0.95],yy+hh*[0.75;0.75],color=color(0,0,255),thickness=0.25);
    xpoly(xx+ww*[0.55;0.95],yy+hh*[0.5;0.5],color=color(0,0,255),thickness=0.25);
    xpoly(xx+ww*[0.55;0.95],yy+hh*[0.25;0.25],color=color(0,0,255),thickness=0.25);
    xfpoly(xx+ww*[0.8;0.7;0.7;0.8],yy+hh*[0.5;0.525;0.475;0.5],color=color(0,0,255),fill_color=color(0,0,255),thickness=0.25);
  endfunction

x=[];y=[];typ=[];

select job
  case 'plot' then
    W=arg1.graphics.exprs(1);
    L=arg1.graphics.exprs(2);
    Beta=arg1.graphics.exprs(3);
    Vt=arg1.graphics.exprs(4);
    K2=arg1.graphics.exprs(5);
    K5=arg1.graphics.exprs(6);
    dW=arg1.graphics.exprs(7);
    dL=arg1.graphics.exprs(8);
    RDS=arg1.graphics.exprs(9);
    standard_draw(arg1,%f,_CEAS_PMOS_dp);
  case 'getinputs' then
    [x,y,typ]=_CEAS_PMOS_ip(arg1);
  case 'getoutputs' then
    [x,y,typ]=_CEAS_PMOS_op(arg1);
  case 'getorigin' then
    [x,y]=standard_origin(arg1);
  case 'set' then
    x=arg1;
    graphics=arg1.graphics;exprs=graphics.exprs;
    model=arg1.model;
    while %t do
      [ok,W,L,Beta,Vt,K2,K5,dW,dL,RDS,exprs]=..
        getvalue(['';'CEAS_PMOS';'';'Simple MOS Transistor';''],..
        [' W [m] : Width',' L [m] : Length',' Beta [A/(V*V)] : Transconductance parameter',' Vt [V] : Zero bias threshold voltage',' K2 [-] : Bulk threshold parameter',' K5 [-] : Reduction of pinch-off region',' dW [m] : Narrowing of channel',' dL [m] : Shortening of channel',' RDS [Ohm] : Drain-Source-Resistance'],..
        list('vec',1,'vec',1,'vec',1,'vec',1,'vec',1,'vec',1,'vec',1,'vec',1,'vec',1),exprs);
      if ~ok then break, end
      model.equations.parameters(2)=list(W,L,Beta,Vt,K2,K5,dW,dL,RDS)
      model.in=[1];
      model.out=[1;1;1];
      graphics.exprs=exprs;
      x.graphics=graphics;x.model=model;
      break
    end
  case 'define' then
    W=0.00002;
    L=0.000006;
    Beta=0.0000105;
    Vt=-1;
    K2=0.41;
    K5=0.839;
    dW=-0.0000025;
    dL=-0.0000021;
    RDS=10000000;
    model=scicos_model();
    model.sim='Coselica';
    model.blocktype='c';
    model.dep_ut=[%t %f];
    model.in=[1];
    model.out=[1;1;1];
    mo=modelica();
      mo.model='Coselica.Electrical.Analog.Semiconductors.PMOS';
      mo.inputs=['G'];
      mo.outputs=['D','B','S'];
      mo.parameters=list(['W','L','Beta','Vt','K2','K5','dW','dL','RDS'],..
                         list(W,L,Beta,Vt,K2,K5,dW,dL,RDS),..
                         [0,0,0,0,0,0,0,0,0]);
    model.equations=mo;
    exprs=[strcat(sci2exp(W));strcat(sci2exp(L));strcat(sci2exp(Beta));strcat(sci2exp(Vt));strcat(sci2exp(K2));strcat(sci2exp(K5));strcat(sci2exp(dW));strcat(sci2exp(dL));strcat(sci2exp(RDS))];
    gr_i="blk_draw(sz,orig,orient,model.label)";
    x=standard_define([2 2],model,exprs,list(gr_i,0));
    x.graphics.in_implicit=['I'];
    x.graphics.out_implicit=['I','I','I'];
  end
endfunction
