// Coselica Toolbox for Scicos/Nsp
// Copyright (C) 2009-2016  Dirk Reusch, Kybernetik Dr. Reusch
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.

function [x,y,typ]=CEAS_PNP(job,arg1,arg2)

  function blk_draw(sz,orig,orient,label)
    if orient then
      xx=orig(1);yy=orig(2);
      ww=sz(1);hh=sz(2);
    else
      xx=orig(1)+sz(1);yy=orig(2);
      ww=-sz(1);hh=sz(2);
    end
    if orient then
      xstring(orig(1)+sz(1)*0,orig(2)+sz(2)*0.9,model.label,w=sz(1)*1,h=sz(2)*0.1,fill=%t,color=color(0,0,255));
    else
      xstring(orig(1)+sz(1)*(0),orig(2)+sz(2)*0.9,model.label,w=sz(1)*1,h=sz(2)*0.1,fill=%t,color=color(0,0,255));
    end
    xpoly(xx+ww*[0.45;0.45],yy+hh*[0.7;0.3],color=color(0,0,255),thickness=0.25);
    xpoly(xx+ww*[0.45;0.05],yy+hh*[0.5;0.5],color=color(0,0,255),thickness=0.25);
    xpoly(xx+ww*[0.95;0.65],yy+hh*[0.75;0.75],color=color(0,0,255),thickness=0.25);
    xpoly(xx+ww*[0.65;0.45],yy+hh*[0.75;0.55],color=color(0,0,255),thickness=0.25);
    xpoly(xx+ww*[0.45;0.65],yy+hh*[0.45;0.25],color=color(0,0,255),thickness=0.25);
    xpoly(xx+ww*[0.65;0.95],yy+hh*[0.25;0.25],color=color(0,0,255),thickness=0.25);
    xfpoly(xx+ww*[0.45;0.525;0.485;0.45],yy+hh*[0.45;0.415;0.375;0.45],color=color(0,0,255),fill_color=color(0,0,255),thickness=0.25);
  endfunction

x=[];y=[];typ=[];

select job
  case 'plot' then
    Bf=arg1.graphics.exprs(1);
    Br=arg1.graphics.exprs(2);
    Is=arg1.graphics.exprs(3);
    Vak=arg1.graphics.exprs(4);
    Tauf=arg1.graphics.exprs(5);
    Taur=arg1.graphics.exprs(6);
    Ccs=arg1.graphics.exprs(7);
    Cje=arg1.graphics.exprs(8);
    Cjc=arg1.graphics.exprs(9);
    Phie=arg1.graphics.exprs(10);
    Me=arg1.graphics.exprs(11);
    Phic=arg1.graphics.exprs(12);
    Mc=arg1.graphics.exprs(13);
    Gbc=arg1.graphics.exprs(14);
    Gbe=arg1.graphics.exprs(15);
    Vt=arg1.graphics.exprs(16);
    EMin=arg1.graphics.exprs(17);
    EMax=arg1.graphics.exprs(18);
    standard_draw(arg1,%f,_CEAS_PNP_dp);
  case 'getinputs' then
    [x,y,typ]=_CEAS_PNP_ip(arg1);
  case 'getoutputs' then
    [x,y,typ]=_CEAS_PNP_op(arg1);
  case 'getorigin' then
    [x,y]=standard_origin(arg1);
  case 'set' then
    x=arg1;
    graphics=arg1.graphics;exprs=graphics.exprs;
    model=arg1.model;
    while %t do
      [ok,Bf,Br,Is,Vak,Tauf,Taur,Ccs,Cje,Cjc,Phie,Me,Phic,Mc,Gbc,Gbe,Vt,EMin,EMax,exprs]=..
        getvalue(['';'CEAS_PNP';'';'Simple BJT according to Ebers-Moll';''],..
        [' Bf [-] : Forward beta',' Br [-] : Reverse beta',' Is [A] : Transport saturation current',' Vak [1/V] : Early voltage (inverse), 1/Volt',' Tauf [s] : Ideal forward transit time',' Taur [s] : Ideal reverse transit time',' Ccs [F] : Collector-substrat(ground) cap.',' Cje [F] : Base-emitter zero bias depletion cap.',' Cjc [F] : Base-coll. zero bias depletion cap.',' Phie [V] : Base-emitter diffusion voltage',' Me [-] : Base-emitter gradation exponent',' Phic [V] : Base-collector diffusion voltage',' Mc [-] : Base-collector gradation exponent',' Gbc [S] : Base-collector conductance',' Gbe [S] : Base-emitter conductance',' Vt [V] : Voltage equivalent of temperature',' EMin [-] : if x < EMin, the exp(x) function is linearized',' EMax [-] : if x > EMax, the exp(x) function is linearized'],..
        list('vec',1,'vec',1,'vec',1,'vec',1,'vec',1,'vec',1,'vec',1,'vec',1,'vec',1,'vec',1,'vec',1,'vec',1,'vec',1,'vec',1,'vec',1,'vec',1,'vec',1,'vec',1),exprs);
      if ~ok then break, end
      model.equations.parameters(2)=list(Bf,Br,Is,Vak,Tauf,Taur,Ccs,Cje,Cjc,Phie,Me,Phic,Mc,Gbc,Gbe,Vt,EMin,EMax)
      model.in=[1];
      model.out=[1;1];
      graphics.exprs=exprs;
      x.graphics=graphics;x.model=model;
      break
    end
  case 'define' then
    Bf=50;
    Br=0.1;
    Is=1.000D-16;
    Vak=0.02;
    Tauf=1.200D-10;
    Taur=5.000D-09;
    Ccs=1.000D-12;
    Cje=4.000D-13;
    Cjc=5.000D-13;
    Phie=0.8;
    Me=0.4;
    Phic=0.8;
    Mc=0.333;
    Gbc=1.000D-15;
    Gbe=1.000D-15;
    Vt=0.02585;
    EMin=-100;
    EMax=40;
    model=scicos_model();
    model.sim='Coselica';
    model.blocktype='c';
    model.dep_ut=[%t %f];
    model.in=[1];
    model.out=[1;1];
    mo=modelica();
      mo.model='Coselica.Electrical.Analog.Semiconductors.PNP';
      mo.inputs=['B'];
      mo.outputs=['C','E'];
      mo.parameters=list(['Bf','Br','Is','Vak','Tauf','Taur','Ccs','Cje','Cjc','Phie','Me','Phic','Mc','Gbc','Gbe','Vt','EMin','EMax'],..
                         list(Bf,Br,Is,Vak,Tauf,Taur,Ccs,Cje,Cjc,Phie,Me,Phic,Mc,Gbc,Gbe,Vt,EMin,EMax),..
                         [0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0]);
    model.equations=mo;
    exprs=[strcat(sci2exp(Bf));strcat(sci2exp(Br));strcat(sci2exp(Is));strcat(sci2exp(Vak));strcat(sci2exp(Tauf));strcat(sci2exp(Taur));strcat(sci2exp(Ccs));strcat(sci2exp(Cje));strcat(sci2exp(Cjc));strcat(sci2exp(Phie));strcat(sci2exp(Me));strcat(sci2exp(Phic));strcat(sci2exp(Mc));strcat(sci2exp(Gbc));strcat(sci2exp(Gbe));strcat(sci2exp(Vt));strcat(sci2exp(EMin));strcat(sci2exp(EMax))];
    gr_i="blk_draw(sz,orig,orient,model.label)";
    x=standard_define([2 2],model,exprs,list(gr_i,0));
    x.graphics.in_implicit=['I'];
    x.graphics.out_implicit=['I','I'];
  end
endfunction
