// Coselica Toolbox for Scicos/Nsp
// Copyright (C) 2009-2016  Dirk Reusch, Kybernetik Dr. Reusch
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.

function [x,y,typ]=CEAS_SineCurrent(job,arg1,arg2)

  function blk_draw(sz,orig,orient,label)
    if orient then
      xx=orig(1);yy=orig(2);
      ww=sz(1);hh=sz(2);
    else
      xx=orig(1)+sz(1);yy=orig(2);
      ww=-sz(1);hh=sz(2);
    end
    if orient then
      xarc(orig(1)+sz(1)*0.25,orig(2)+sz(2)*0.75,sz(1)*0.5,sz(2)*0.5,0,23040,color=color(0,0,0),background=color(255,255,255),thickness=0.25);
    else
      xarc(orig(1)+sz(1)*(1-0.25-0.5),orig(2)+sz(2)*0.75,sz(1)*0.5,sz(2)*0.5,-11520,23040,color=color(0,0,0),background=color(255,255,255),thickness=0.25);
    end
    xpoly(xx+ww*[0.05;0.25],yy+hh*[0.5;0.5],color=color(0,0,0),thickness=0.25);
    xpoly(xx+ww*[0.75;0.95],yy+hh*[0.5;0.5],color=color(0,0,0),thickness=0.25);
    xpoly(xx+ww*[0.5;0.5],yy+hh*[0.25;0.75],color=color(0,0,0),thickness=0.25);
    if orient then
      xstring(orig(1)+sz(1)*-0.25,orig(2)+sz(2)*0.9,model.label,w=sz(1)*1.5,h=sz(2)*0.2,fill=%t,color=color(0,0,255));
    else
      xstring(orig(1)+sz(1)*(-0.25),orig(2)+sz(2)*0.9,model.label,w=sz(1)*1.5,h=sz(2)*0.2,fill=%t,color=color(0,0,255));
    end
    xfpoly(xx+ww*[0.95;0.8;0.8;0.95],yy+hh*[0.5;0.55;0.45;0.5],color=color(0,0,255),fill_color=color(0,0,255),thickness=0.25);
    xpoly(xx+ww*[0.15;0.199;0.231;0.259;0.2835;0.3085;0.333;0.3575;0.382;0.407;0.435;0.4701;0.54395;0.5755;0.604;0.6285;0.653;0.6775;0.7025;0.727;0.7515;0.7795;0.815;0.85],yy+hh*[0.5;0.6495;0.7325;0.7905;0.826;0.846;0.849;0.835;0.805;0.76;0.693;0.593;0.3655;0.28;0.219;0.18;0.157;0.15;0.1605;0.1875;0.2295;0.2935;0.3915;0.5],color=color(192,192,192),thickness=0.25);
  endfunction

x=[];y=[];typ=[];

select job
  case 'plot' then
    I=arg1.graphics.exprs(1);
    phase=arg1.graphics.exprs(2);
    freqHz=arg1.graphics.exprs(3);
    offset=arg1.graphics.exprs(4);
    startTime=arg1.graphics.exprs(5);
    standard_draw(arg1,%f,_MEAI_OnePort_dp);
  case 'getinputs' then
    [x,y,typ]=_MEAI_OnePort_ip(arg1);
  case 'getoutputs' then
    [x,y,typ]=_MEAI_OnePort_op(arg1);
  case 'getorigin' then
    [x,y]=standard_origin(arg1);
  case 'set' then
    x=arg1;
    graphics=arg1.graphics;exprs=graphics.exprs;
    model=arg1.model;
    while %t do
      [ok,I,phase,freqHz,offset,startTime,exprs]=..
        getvalue(['';'CEAS_SineCurrent';'';'Sine current source';''],..
        [' I [A] : Amplitude of sine wave',' phase [rad] : Phase of sine wave',' freqHz [Hz] : Frequency of sine wave',' offset [A] : Current offset',' startTime [s] : Time offset'],..
        list('vec',1,'vec',1,'vec',1,'vec',1,'vec',1),exprs);
      if ~ok then break, end
      model.equations.parameters(2)=list(I,phase,freqHz,offset,startTime)
      model.in=[1];
      model.out=[1];
      graphics.exprs=exprs;
      x.graphics=graphics;x.model=model;
      break
    end
  case 'define' then
    I=1;
    phase=0;
    freqHz=1;
    offset=0;
    startTime=0;
    model=scicos_model();
    model.sim='Coselica';
    model.blocktype='c';
    model.dep_ut=[%t %f];
    model.in=[1];
    model.out=[1];
    mo=modelica();
      mo.model='Coselica.Electrical.Analog.Sources.SineCurrent';
      mo.inputs=['p'];
      mo.outputs=['n'];
      mo.parameters=list(['I','phase','freqHz','offset','startTime'],..
                         list(I,phase,freqHz,offset,startTime),..
                         [0,0,0,0,0]);
    model.equations=mo;
    exprs=[strcat(sci2exp(I));strcat(sci2exp(phase));strcat(sci2exp(freqHz));strcat(sci2exp(offset));strcat(sci2exp(startTime))];
    gr_i="blk_draw(sz,orig,orient,model.label)";
    x=standard_define([2 2],model,exprs,list(gr_i,0));
    x.graphics.in_implicit=['I'];
    x.graphics.out_implicit=['I'];
  end
endfunction
