// Coselica Toolbox for Scicos/Nsp
// Copyright (C) 2009-2016  Dirk Reusch, Kybernetik Dr. Reusch
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.

function [x,y,typ]=CEAS_SineVoltage(job,arg1,arg2)

  function blk_draw(sz,orig,orient,label)
    if orient then
      xx=orig(1);yy=orig(2);
      ww=sz(1);hh=sz(2);
    else
      xx=orig(1)+sz(1);yy=orig(2);
      ww=-sz(1);hh=sz(2);
    end
    if orient then
      xarc(orig(1)+sz(1)*0.25,orig(2)+sz(2)*0.75,sz(1)*0.5,sz(2)*0.5,0,23040,color=color(0,0,0),background=color(255,255,255),thickness=0.25);
    else
      xarc(orig(1)+sz(1)*(1-0.25-0.5),orig(2)+sz(2)*0.75,sz(1)*0.5,sz(2)*0.5,-11520,23040,color=color(0,0,0),background=color(255,255,255),thickness=0.25);
    end
    if orient then
      xstring(orig(1)+sz(1)*-0.25,orig(2)+sz(2)*0.9,model.label,w=sz(1)*1.5,h=sz(2)*0.2,fill=%t,color=color(0,0,255));
    else
      xstring(orig(1)+sz(1)*(-0.25),orig(2)+sz(2)*0.9,model.label,w=sz(1)*1.5,h=sz(2)*0.2,fill=%t,color=color(0,0,255));
    end
    xpoly(xx+ww*[0.05;0.95],yy+hh*[0.5;0.5],color=color(0,0,0),thickness=0.25);
    if orient then
      xstring(orig(1)+sz(1)*-0.1,orig(2)+sz(2)*0.55,"+",w=sz(1)*0.5,h=sz(2)*0.15,fill=%t,color=color(0,0,255));
    else
      xstring(orig(1)+sz(1)*(0.6),orig(2)+sz(2)*0.55,"+",w=sz(1)*0.5,h=sz(2)*0.15,fill=%t,color=color(0,0,255));
    end
    if orient then
      xstring(orig(1)+sz(1)*0.6,orig(2)+sz(2)*0.5,"-",w=sz(1)*0.5,h=sz(2)*0.25,fill=%t,color=color(0,0,255));
    else
      xstring(orig(1)+sz(1)*(-0.1),orig(2)+sz(2)*0.5,"-",w=sz(1)*0.5,h=sz(2)*0.25,fill=%t,color=color(0,0,255));
    end
    xpoly(xx+ww*[0.17;0.219;0.251;0.279;0.3035;0.3285;0.353;0.3775;0.402;0.427;0.455;0.4901;0.56395;0.5955;0.624;0.6485;0.673;0.6975;0.7225;0.747;0.7715;0.7995;0.835;0.87],yy+hh*[0.5;0.6495;0.7325;0.7905;0.826;0.846;0.849;0.835;0.805;0.76;0.693;0.593;0.3655;0.28;0.219;0.18;0.157;0.15;0.1605;0.1875;0.2295;0.2935;0.3915;0.5],color=color(192,192,192),thickness=0.25);
  endfunction

x=[];y=[];typ=[];

select job
  case 'plot' then
    V=arg1.graphics.exprs(1);
    phase=arg1.graphics.exprs(2);
    freqHz=arg1.graphics.exprs(3);
    offset=arg1.graphics.exprs(4);
    startTime=arg1.graphics.exprs(5);
    standard_draw(arg1,%f,_MEAI_OnePort_dp);
  case 'getinputs' then
    [x,y,typ]=_MEAI_OnePort_ip(arg1);
  case 'getoutputs' then
    [x,y,typ]=_MEAI_OnePort_op(arg1);
  case 'getorigin' then
    [x,y]=standard_origin(arg1);
  case 'set' then
    x=arg1;
    graphics=arg1.graphics;exprs=graphics.exprs;
    model=arg1.model;
    while %t do
      [ok,V,phase,freqHz,offset,startTime,exprs]=..
        getvalue(['';'CEAS_SineVoltage';'';'Sine voltage source';''],..
        [' V [V] : Amplitude of sine wave',' phase [rad] : Phase of sine wave',' freqHz [Hz] : Frequency of sine wave',' offset [V] : Voltage offset',' startTime [s] : Time offset'],..
        list('vec',1,'vec',1,'vec',1,'vec',1,'vec',1),exprs);
      if ~ok then break, end
      model.equations.parameters(2)=list(V,phase,freqHz,offset,startTime)
      model.in=[1];
      model.out=[1];
      graphics.exprs=exprs;
      x.graphics=graphics;x.model=model;
      break
    end
  case 'define' then
    V=1;
    phase=0;
    freqHz=1;
    offset=0;
    startTime=0;
    model=scicos_model();
    model.sim='Coselica';
    model.blocktype='c';
    model.dep_ut=[%t %f];
    model.in=[1];
    model.out=[1];
    mo=modelica();
      mo.model='Coselica.Electrical.Analog.Sources.SineVoltage';
      mo.inputs=['p'];
      mo.outputs=['n'];
      mo.parameters=list(['V','phase','freqHz','offset','startTime'],..
                         list(V,phase,freqHz,offset,startTime),..
                         [0,0,0,0,0]);
    model.equations=mo;
    exprs=[strcat(sci2exp(V));strcat(sci2exp(phase));strcat(sci2exp(freqHz));strcat(sci2exp(offset));strcat(sci2exp(startTime))];
    gr_i="blk_draw(sz,orig,orient,model.label)";
    x=standard_define([2 2],model,exprs,list(gr_i,0));
    x.graphics.in_implicit=['I'];
    x.graphics.out_implicit=['I'];
  end
endfunction
