// Coselica Toolbox for Scicos/Nsp
// Copyright (C) 2009-2016  Dirk Reusch, Kybernetik Dr. Reusch
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.

function [x,y,typ]=CEAS_Thyristor(job,arg1,arg2)

  function blk_draw(sz,orig,orient,label)
    if orient then
      xx=orig(1);yy=orig(2);
      ww=sz(1);hh=sz(2);
    else
      xx=orig(1)+sz(1);yy=orig(2);
      ww=-sz(1);hh=sz(2);
    end
    xfpoly(xx+ww*[0.65;0.35;0.35;0.65],yy+hh*[0.5;0.7;0.3;0.5],color=color(0,0,255),fill_color=color(255,255,255),thickness=0.25);
    xpoly(xx+ww*[0.05;0.7],yy+hh*[0.5;0.5],color=color(0,0,255),thickness=0.25);
    xpoly(xx+ww*[0.7;0.95],yy+hh*[0.5;0.5],color=color(0,0,255),thickness=0.25);
    xpoly(xx+ww*[0.65;0.65],yy+hh*[0.7;0.3],color=color(0,0,255),thickness=0.25);
    xpoly(xx+ww*[0.65;0.85;0.85],yy+hh*[0.6;0.8;0.95],color=color(0,0,255),thickness=0.25);
    xpoly(xx+ww*[0.7;0.8],yy+hh*[0.75;0.65],color=color(0,0,255),thickness=0.25);
    if orient then
      xstring(orig(1)+sz(1)*-0.21,orig(2)+sz(2)*0.005,model.label,w=sz(1)*1.5,h=sz(2)*0.2,fill=%t,color=color(0,0,255));
    else
      xstring(orig(1)+sz(1)*(-0.29),orig(2)+sz(2)*0.005,model.label,w=sz(1)*1.5,h=sz(2)*0.2,fill=%t,color=color(0,0,255));
    end
  endfunction

x=[];y=[];typ=[];

select job
  case 'plot' then
    VDRM=arg1.graphics.exprs(1);
    VRRM=arg1.graphics.exprs(2);
    IDRM=arg1.graphics.exprs(3);
    VTM=arg1.graphics.exprs(4);
    IH=arg1.graphics.exprs(5);
    ITM=arg1.graphics.exprs(6);
    VGT=arg1.graphics.exprs(7);
    IGT=arg1.graphics.exprs(8);
    TON=arg1.graphics.exprs(9);
    TOFF=arg1.graphics.exprs(10);
    Vt=arg1.graphics.exprs(11);
    Nbv=arg1.graphics.exprs(12);
    standard_draw(arg1,%f,_CEAS_Thyristor_dp);
  case 'getinputs' then
    [x,y,typ]=_CEAS_Thyristor_ip(arg1);
  case 'getoutputs' then
    [x,y,typ]=_CEAS_Thyristor_op(arg1);
  case 'getorigin' then
    [x,y]=standard_origin(arg1);
  case 'set' then
    x=arg1;
    graphics=arg1.graphics;exprs=graphics.exprs;
    model=arg1.model;
    while %t do
      [ok,VDRM,VRRM,IDRM,VTM,IH,ITM,VGT,IGT,TON,TOFF,Vt,Nbv,exprs]=..
        getvalue(['';'CEAS_Thyristor';'';'Simple Thyristor Model';''],..
        [' VDRM [V] : Forward breakthrough voltage (>= 0)',' VRRM [V] : Reverse breakthrough voltage (>= 0)',' IDRM [A] : Saturation current',' VTM [V] : Conducting voltage',' IH [A] : Holding current',' ITM [A] : Conducting current',' VGT [V] : Gate trigger voltage',' IGT [A] : Gate trigger current',' TON [s] : Switch on time',' TOFF [s] : Switch off time',' Vt [V] : Voltage equivalent of temperature (kT/qn)',' Nbv [-] : Reverse Breakthrough emission coefficient'],..
        list('vec',1,'vec',1,'vec',1,'vec',1,'vec',1,'vec',1,'vec',1,'vec',1,'vec',1,'vec',1,'vec',1,'vec',1),exprs);
      if ~ok then break, end
      model.equations.parameters(2)=list(VDRM,VRRM,IDRM,VTM,IH,ITM,VGT,IGT,TON,TOFF,Vt,Nbv)
      model.in=[1;1];
      model.out=[1];
      graphics.exprs=exprs;
      x.graphics=graphics;x.model=model;
      break
    end
  case 'define' then
    VDRM=100;
    VRRM=100;
    IDRM=0.1;
    VTM=1.7;
    IH=0.006;
    ITM=25;
    VGT=0.7;
    IGT=0.005;
    TON=0.000001;
    TOFF=0.000015;
    Vt=0.04;
    Nbv=0.74;
    model=scicos_model();
    model.sim='Coselica';
    model.blocktype='c';
    model.dep_ut=[%t %f];
    model.in=[1;1];
    model.out=[1];
    mo=modelica();
      mo.model='Coselica.Electrical.Analog.Semiconductors.Thyristor';
      mo.inputs=['Anode','Gate'];
      mo.outputs=['Cathode'];
      mo.parameters=list(['VDRM','VRRM','IDRM','VTM','IH','ITM','VGT','IGT','TON','TOFF','Vt','Nbv'],..
                         list(VDRM,VRRM,IDRM,VTM,IH,ITM,VGT,IGT,TON,TOFF,Vt,Nbv),..
                         [0,0,0,0,0,0,0,0,0,0,0,0]);
    model.equations=mo;
    exprs=[strcat(sci2exp(VDRM));strcat(sci2exp(VRRM));strcat(sci2exp(IDRM));strcat(sci2exp(VTM));strcat(sci2exp(IH));strcat(sci2exp(ITM));strcat(sci2exp(VGT));strcat(sci2exp(IGT));strcat(sci2exp(TON));strcat(sci2exp(TOFF));strcat(sci2exp(Vt));strcat(sci2exp(Nbv))];
    gr_i="blk_draw(sz,orig,orient,model.label)";
    x=standard_define([2 2],model,exprs,list(gr_i,0));
    x.graphics.in_implicit=['I','I'];
    x.graphics.out_implicit=['I'];
  end
endfunction
