// Coselica Toolbox for Scicos/Nsp
// Copyright (C) 2009-2016  Dirk Reusch, Kybernetik Dr. Reusch
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.

function [x,y,typ]=CEAS_ZDiode(job,arg1,arg2)

  function blk_draw(sz,orig,orient,label)
    if orient then
      xx=orig(1);yy=orig(2);
      ww=sz(1);hh=sz(2);
    else
      xx=orig(1)+sz(1);yy=orig(2);
      ww=-sz(1);hh=sz(2);
    end
    xfpoly(xx+ww*[0.65;0.35;0.35;0.65],yy+hh*[0.5;0.7;0.3;0.5],color=color(0,0,255),fill_color=color(255,255,255),thickness=0.25);
    xpoly(xx+ww*[0.05;0.7],yy+hh*[0.5;0.5],color=color(0,0,255),thickness=0.25);
    xpoly(xx+ww*[0.7;0.95],yy+hh*[0.5;0.5],color=color(0,0,255),thickness=0.25);
    xpoly(xx+ww*[0.65;0.65],yy+hh*[0.7;0.3],color=color(0,0,255),thickness=0.25);
    if orient then
      xstring(orig(1)+sz(1)*-0.15,orig(2)+sz(2)*0,"Bv="+string(Bv)+"",w=sz(1)*1.3,h=sz(2)*0.2,fill=%t,color=color(0,0,0));
    else
      xstring(orig(1)+sz(1)*(-0.15),orig(2)+sz(2)*0,"Bv="+string(Bv)+"",w=sz(1)*1.3,h=sz(2)*0.2,fill=%t,color=color(0,0,0));
    end
    xpoly(xx+ww*[0.65;0.6],yy+hh*[0.3;0.3],color=color(0,0,255),thickness=0.25);
    if orient then
      xstring(orig(1)+sz(1)*-0.25,orig(2)+sz(2)*0.78,model.label,w=sz(1)*1.5,h=sz(2)*0.2,fill=%t,color=color(0,0,255));
    else
      xstring(orig(1)+sz(1)*(-0.25),orig(2)+sz(2)*0.78,model.label,w=sz(1)*1.5,h=sz(2)*0.2,fill=%t,color=color(0,0,255));
    end
  endfunction

x=[];y=[];typ=[];

select job
  case 'plot' then
    Ids=arg1.graphics.exprs(1);
    Vt=arg1.graphics.exprs(2);
    Maxexp=arg1.graphics.exprs(3);
    R=arg1.graphics.exprs(4);
    Bv=arg1.graphics.exprs(5);
    Ibv=arg1.graphics.exprs(6);
    Nbv=arg1.graphics.exprs(7);
    standard_draw(arg1,%f,_MEAI_OnePort_dp);
  case 'getinputs' then
    [x,y,typ]=_MEAI_OnePort_ip(arg1);
  case 'getoutputs' then
    [x,y,typ]=_MEAI_OnePort_op(arg1);
  case 'getorigin' then
    [x,y]=standard_origin(arg1);
  case 'set' then
    x=arg1;
    graphics=arg1.graphics;exprs=graphics.exprs;
    model=arg1.model;
    while %t do
      [ok,Ids,Vt,Maxexp,R,Bv,Ibv,Nbv,exprs]=..
        getvalue(['';'CEAS_ZDiode';'';'Zener Diode with 3 working areas';''],..
        [' Ids [A] : Saturation current',' Vt [V] : Voltage equivalent of temperature (kT/qn)',' Maxexp [-] : Max. exponent for linear continuation',' R [Ohm] : Parallel ohmic resistance',' Bv [V] : Breakthrough voltage = Zener- or Z-voltage',' Ibv [A] : Breakthrough knee current',' Nbv [-] : Breakthrough emission coefficient'],..
        list('vec',1,'vec',1,'vec',1,'vec',1,'vec',1,'vec',1,'vec',1),exprs);
      if ~ok then break, end
      model.equations.parameters(2)=list(Ids,Vt,Maxexp,R,Bv,Ibv,Nbv)
      model.in=[1];
      model.out=[1];
      graphics.exprs=exprs;
      x.graphics=graphics;x.model=model;
      break
    end
  case 'define' then
    Ids=0.000001;
    Vt=0.04;
    Maxexp=30;
    R=1.000D+08;
    Bv=5.1;
    Ibv=0.7;
    Nbv=0.74;
    model=scicos_model();
    model.sim='Coselica';
    model.blocktype='c';
    model.dep_ut=[%t %f];
    model.in=[1];
    model.out=[1];
    mo=modelica();
      mo.model='Coselica.Electrical.Analog.Semiconductors.ZDiode';
      mo.inputs=['p'];
      mo.outputs=['n'];
      mo.parameters=list(['Ids','Vt','Maxexp','R','Bv','Ibv','Nbv'],..
                         list(Ids,Vt,Maxexp,R,Bv,Ibv,Nbv),..
                         [0,0,0,0,0,0,0]);
    model.equations=mo;
    exprs=[strcat(sci2exp(Ids));strcat(sci2exp(Vt));strcat(sci2exp(Maxexp));strcat(sci2exp(R));strcat(sci2exp(Bv));strcat(sci2exp(Ibv));strcat(sci2exp(Nbv))];
    gr_i="blk_draw(sz,orig,orient,model.label)";
    x=standard_define([2 2],model,exprs,list(gr_i,0));
    x.graphics.in_implicit=['I'];
    x.graphics.out_implicit=['I'];
  end
endfunction
