// Coselica Toolbox for Scicos/Nsp
// Copyright (C) 2009-2016  Dirk Reusch, Kybernetik Dr. Reusch
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.

function [x,y,typ]=CEMB_Resistor(job,arg1,arg2)

  function blk_draw(sz,orig,orient,label)
    if orient then
      xx=orig(1);yy=orig(2);
      ww=sz(1);hh=sz(2);
    else
      xx=orig(1)+sz(1);yy=orig(2);
      ww=-sz(1);hh=sz(2);
    end
    if orient then
      xrect(orig(1)+sz(1)*0.15,orig(2)+sz(2)*0.65,sz(1)*0.7,sz(2)*0.3,color=color(0,0,255),background=color(255,255,255),thickness=0.25);
    else
      xrect(orig(1)+sz(1)*(1-0.15-0.7),orig(2)+sz(2)*0.65,sz(1)*0.7,sz(2)*0.3,color=color(0,0,255),background=color(255,255,255),thickness=0.25);
    end
    xpoly(xx+ww*[0.05;0.15],yy+hh*[0.5;0.5],color=color(0,0,255),thickness=0.25);
    xpoly(xx+ww*[0.85;0.95],yy+hh*[0.5;0.5],color=color(0,0,255),thickness=0.25);
    if orient then
      xstring(orig(1)+sz(1)*-0.25,orig(2)+sz(2)*0.7,model.label,w=sz(1)*1.5,h=sz(2)*0.3,fill=%t,color=color(0,0,255));
    else
      xstring(orig(1)+sz(1)*(-0.25),orig(2)+sz(2)*0.7,model.label,w=sz(1)*1.5,h=sz(2)*0.3,fill=%t,color=color(0,0,255));
    end
    if orient then
      xstring(orig(1)+sz(1)*0,orig(2)+sz(2)*0,"m="+string(m)+"",w=sz(1)*1,h=sz(2)*0.2,fill=%t,color=color(0,0,0));
    else
      xstring(orig(1)+sz(1)*(0),orig(2)+sz(2)*0,"m="+string(m)+"",w=sz(1)*1,h=sz(2)*0.2,fill=%t,color=color(0,0,0));
    end
  endfunction

x=[];y=[];typ=[];

select job
  case 'plot' then
    R=arg1.graphics.exprs(1);
    m=arg1.graphics.exprs(2);
    m=strcat(sci2exp(double(evstr(m))));
    standard_draw(arg1,%f,_CEMI_TwoPlug_dp);
  case 'getinputs' then
    [x,y,typ]=_CEMI_TwoPlug_ip(arg1);
  case 'getoutputs' then
    [x,y,typ]=_CEMI_TwoPlug_op(arg1);
  case 'getorigin' then
    [x,y]=standard_origin(arg1);
  case 'set' then
    x=arg1;
    graphics=arg1.graphics;exprs=graphics.exprs;
    model=arg1.model;
    while %t do
      [ok,R,m,exprs]=..
        getvalue(['';'CEMB_Resistor';'';'Ideal linear electrical resistors';''],..
        [' R [Ohm] : Resistance',' m [-] : number of phases (>1)'],..
        list('vec',-1,'vec',1),exprs);
      if ~ok then break, end
      model.equations.parameters(2)=list(R,m)
      m=double(m);
      model.in=[1];
      model.out=[1];
      graphics.exprs=exprs;
      x.graphics=graphics;x.model=model;
      break
    end
  case 'define' then
    R=[1,1,1];
    m=3;
    model=scicos_model();
    model.sim='Coselica';
    model.blocktype='c';
    model.dep_ut=[%t %f];
    model.in=[1];
    model.out=[1];
    m=int32(3);
    mo=modelica();
      mo.model='Coselica.Electrical.MultiPhase.Basic.Resistor';
      mo.inputs=['plug_p'];
      mo.outputs=['plug_n'];
      mo.parameters=list(['R','m'],..
                         list(R,m),..
                         [0,0]);
    model.equations=mo;
    exprs=[strcat(sci2exp(R));strcat(sci2exp(m))];
    gr_i="blk_draw(sz,orig,orient,model.label)";
    x=standard_define([2 2],model,exprs,list(gr_i,0));
    x.graphics.in_implicit=['I'];
    x.graphics.out_implicit=['I'];
  end
endfunction
