// Coselica Toolbox for Scicos/Nsp
// Copyright (C) 2009-2016  Dirk Reusch, Kybernetik Dr. Reusch
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.

function [x,y,typ]=CEMI_IdealDiode(job,arg1,arg2)

  function blk_draw(sz,orig,orient,label)
    if orient then
      xx=orig(1);yy=orig(2);
      ww=sz(1);hh=sz(2);
    else
      xx=orig(1)+sz(1);yy=orig(2);
      ww=-sz(1);hh=sz(2);
    end
    if orient then
      xstring(orig(1)+sz(1)*-0.25,orig(2)+sz(2)*0,model.label,w=sz(1)*1.5,h=sz(2)*0.3,fill=%t,color=color(0,0,255));
    else
      xstring(orig(1)+sz(1)*(-0.25),orig(2)+sz(2)*0,model.label,w=sz(1)*1.5,h=sz(2)*0.3,fill=%t,color=color(0,0,255));
    end
    xpoly(xx+ww*[0.05;0.7],yy+hh*[0.5;0.5],color=color(0,0,255),thickness=0.25);
    xpoly(xx+ww*[0.65;0.35;0.35;0.65],yy+hh*[0.5;0.7;0.3;0.5],color=color(0,0,255),thickness=0.25);
    xpoly(xx+ww*[0.65;0.65],yy+hh*[0.7;0.3],color=color(0,0,255),thickness=0.25);
    xpoly(xx+ww*[0.7;0.95],yy+hh*[0.5;0.5],color=color(0,0,255),thickness=0.25);
    if orient then
      xstring(orig(1)+sz(1)*0.1,orig(2)+sz(2)*0.8,"m="+string(m)+"",w=sz(1)*0.8,h=sz(2)*0.2,fill=%t,color=color(0,0,0));
    else
      xstring(orig(1)+sz(1)*(0.1),orig(2)+sz(2)*0.8,"m="+string(m)+"",w=sz(1)*0.8,h=sz(2)*0.2,fill=%t,color=color(0,0,0));
    end
  endfunction

x=[];y=[];typ=[];

select job
  case 'plot' then
    Ron=arg1.graphics.exprs(1);
    Goff=arg1.graphics.exprs(2);
    Vknee=arg1.graphics.exprs(3);
    m=arg1.graphics.exprs(4);
    m=strcat(sci2exp(double(evstr(m))));
    standard_draw(arg1,%f,_CEMI_TwoPlug_dp);
  case 'getinputs' then
    [x,y,typ]=_CEMI_TwoPlug_ip(arg1);
  case 'getoutputs' then
    [x,y,typ]=_CEMI_TwoPlug_op(arg1);
  case 'getorigin' then
    [x,y]=standard_origin(arg1);
  case 'set' then
    x=arg1;
    graphics=arg1.graphics;exprs=graphics.exprs;
    model=arg1.model;
    while %t do
      [ok,Ron,Goff,Vknee,m,exprs]=..
        getvalue(['';'CEMI_IdealDiode';'';'Multiphase ideal diode';''],..
        [' Ron [Ohm] : Closed diode resistance',' Goff [S] : Opened diode conductance',' Vknee [V] : Treshold voltage',' m [-] : number of phases (>1)'],..
        list('vec',-1,'vec',-1,'vec',-1,'vec',1),exprs);
      if ~ok then break, end
      model.equations.parameters(2)=list(Ron,Goff,Vknee,m)
      m=double(m);
      model.in=[1];
      model.out=[1];
      graphics.exprs=exprs;
      x.graphics=graphics;x.model=model;
      break
    end
  case 'define' then
    Ron=[0.00001,0.00001,0.00001];
    Goff=[0.00001,0.00001,0.00001];
    Vknee=[0,0,0];
    m=3;
    model=scicos_model();
    model.sim='Coselica';
    model.blocktype='c';
    model.dep_ut=[%t %f];
    model.in=[1];
    model.out=[1];
    m=int32(3);
    mo=modelica();
      mo.model='Coselica.Electrical.MultiPhase.Ideal.IdealDiode';
      mo.inputs=['plug_p'];
      mo.outputs=['plug_n'];
      mo.parameters=list(['Ron','Goff','Vknee','m'],..
                         list(Ron,Goff,Vknee,m),..
                         [0,0,0,0]);
    model.equations=mo;
    exprs=[strcat(sci2exp(Ron));strcat(sci2exp(Goff));strcat(sci2exp(Vknee));strcat(sci2exp(m))];
    gr_i="blk_draw(sz,orig,orient,model.label)";
    x=standard_define([2 2],model,exprs,list(gr_i,0));
    x.graphics.in_implicit=['I'];
    x.graphics.out_implicit=['I'];
  end
endfunction
