// Coselica Toolbox for Scicos/Nsp
// Copyright (C) 2009-2016  Dirk Reusch, Kybernetik Dr. Reusch
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.

function [x,y,typ]=CEMS_VoltageSensor(job,arg1,arg2)

  function blk_draw(sz,orig,orient,label)
    if orient then
      xx=orig(1);yy=orig(2);
      ww=sz(1);hh=sz(2);
    else
      xx=orig(1)+sz(1);yy=orig(2);
      ww=-sz(1);hh=sz(2);
    end
    if orient then
      xarc(orig(1)+sz(1)*0.15,orig(2)+sz(2)*0.85,sz(1)*0.7,sz(2)*0.7,0,23040,color=color(0,0,0),background=color(255,255,255),thickness=0.25);
    else
      xarc(orig(1)+sz(1)*(1-0.15-0.7),orig(2)+sz(2)*0.85,sz(1)*0.7,sz(2)*0.7,-11520,23040,color=color(0,0,0),background=color(255,255,255),thickness=0.25);
    end
    xpoly(xx+ww*[0.5;0.5],yy+hh*[0.85;0.7],color=color(0,0,0),thickness=0.25);
    xpoly(xx+ww*[0.6145;0.701],yy+hh*[0.664;0.7865],color=color(0,0,0),thickness=0.25);
    xpoly(xx+ww*[0.3855;0.299],yy+hh*[0.664;0.7865],color=color(0,0,0),thickness=0.25);
    xpoly(xx+ww*[0.688;0.829],yy+hh*[0.5685;0.6195],color=color(0,0,0),thickness=0.25);
    xpoly(xx+ww*[0.312;0.171],yy+hh*[0.5685;0.6195],color=color(0,0,0),thickness=0.25);
    xpoly(xx+ww*[0.5;0.5451],yy+hh*[0.5;0.643],color=color(0,0,0),thickness=0.25);
    xfpoly(xx+ww*[0.4976;0.59;0.59;0.4976],yy+hh*[0.658;0.63;0.786;0.658],color=color(0,0,0),fill_color=color(0,0,0),thickness=0.25);
    if orient then
      xarc(orig(1)+sz(1)*0.475,orig(2)+sz(2)*0.525,sz(1)*0.05,sz(2)*0.05,0,23040,color=color(0,0,0),background=color(0,0,0),thickness=0.25);
    else
      xarc(orig(1)+sz(1)*(1-0.475-0.05),orig(2)+sz(2)*0.525,sz(1)*0.05,sz(2)*0.05,-11520,23040,color=color(0,0,0),background=color(0,0,0),thickness=0.25);
    end
    if orient then
      xstring(orig(1)+sz(1)*0.355,orig(2)+sz(2)*0.15,"V",w=sz(1)*0.295,h=sz(2)*0.295,fill=%t,color=color(0,0,0));
    else
      xstring(orig(1)+sz(1)*(0.35),orig(2)+sz(2)*0.15,"V",w=sz(1)*0.295,h=sz(2)*0.295,fill=%t,color=color(0,0,0));
    end
    if orient then
      xstring(orig(1)+sz(1)*0,orig(2)+sz(2)*0.8,model.label,w=sz(1)*1,h=sz(2)*0.3,fill=%t,color=color(0,0,255));
    else
      xstring(orig(1)+sz(1)*(0),orig(2)+sz(2)*0.8,model.label,w=sz(1)*1,h=sz(2)*0.3,fill=%t,color=color(0,0,255));
    end
    if orient then
      xstring(orig(1)+sz(1)*-0.1,orig(2)+sz(2)*0,"m="+string(m)+"",w=sz(1)*0.5,h=sz(2)*0.2,fill=%t,color=color(0,0,0));
    else
      xstring(orig(1)+sz(1)*(0.6),orig(2)+sz(2)*0,"m="+string(m)+"",w=sz(1)*0.5,h=sz(2)*0.2,fill=%t,color=color(0,0,0));
    end
    xpoly(xx+ww*[0.15;0.05],yy+hh*[0.5;0.5],color=color(0,0,0),thickness=0.25);
    xpoly(xx+ww*[0.85;0.95],yy+hh*[0.5;0.5],color=color(0,0,0),thickness=0.25);
    xpoly(xx+ww*[0.5;0.5],yy+hh*[0;0.15],color=color(0,0,255),thickness=0.25);
  endfunction

x=[];y=[];typ=[];

select job
  case 'plot' then
    m=arg1.graphics.exprs(1);
    m=strcat(sci2exp(double(evstr(m))));
    standard_draw(arg1,%f,_CEMS_VoltageSensor_dp);
  case 'getinputs' then
    [x,y,typ]=_CEMS_VoltageSensor_ip(arg1);
  case 'getoutputs' then
    [x,y,typ]=_CEMS_VoltageSensor_op(arg1);
  case 'getorigin' then
    [x,y]=standard_origin(arg1);
  case 'set' then
    x=arg1;
    graphics=arg1.graphics;exprs=graphics.exprs;
    model=arg1.model;
    while %t do
      [ok,m,exprs]=..
        getvalue(['';'CEMS_VoltageSensor';'';'Multiphase voltage sensor';''],..
        [' m [-] : number of phases (>1)'],..
        list('vec',1),exprs);
      if ~ok then break, end
      model.equations.parameters(2)=list(m)
      m=double(m);
      model.in=[1];
      model.out=[1;m];
      graphics.exprs=exprs;
      x.graphics=graphics;x.model=model;
      break
    end
  case 'define' then
    m=3;
    model=scicos_model();
    model.sim='Coselica';
    model.blocktype='c';
    model.dep_ut=[%t %f];
    model.in=[1];
    model.out=[1;m];
    m=int32(3);
    mo=modelica();
      mo.model='Coselica.Electrical.MultiPhase.Sensors.VoltageSensor';
      mo.inputs=['plug_p'];
      mo.outputs=['plug_n','v'];
      mo.parameters=list(['m'],..
                         list(m),..
                         [0]);
    model.equations=mo;
    exprs=[strcat(sci2exp(m))];
    gr_i="blk_draw(sz,orig,orient,model.label)";
    x=standard_define([2 2],model,exprs,list(gr_i,0));
    x.graphics.in_implicit=['I'];
    x.graphics.out_implicit=['I','I'];
  end
endfunction
