// Coselica Toolbox for Scicos/Nsp
// Copyright (C) 2009-2016  Dirk Reusch, Kybernetik Dr. Reusch
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.

function [x,y,typ]=CMRC_ElastoBacklash(job,arg1,arg2)

  function blk_draw(sz,orig,orient,label)
    if orient then
      xx=orig(1);yy=orig(2);
      ww=sz(1);hh=sz(2);
    else
      xx=orig(1)+sz(1);yy=orig(2);
      ww=-sz(1);hh=sz(2);
    end
    xpoly(xx+ww*[0.1;0.21;0.26;0.33;0.4;0.46;0.5;0.6],yy+hh*[0.66;0.66;0.5;0.805;0.5;0.8;0.65;0.65],color=color(0,0,0),thickness=0.25);
    if orient then
      xrect(orig(1)+sz(1)*0.2,orig(2)+sz(2)*0.45,sz(1)*0.25,sz(2)*0.2,color=color(0,0,0),background=color(192,192,192),thickness=0.25);
    else
      xrect(orig(1)+sz(1)*(1-0.2-0.25),orig(2)+sz(2)*0.45,sz(1)*0.25,sz(2)*0.2,color=color(0,0,0),background=color(192,192,192),thickness=0.25);
    end
    xpoly(xx+ww*[0.2;0.5],yy+hh*[0.25;0.25],color=color(0,0,0),thickness=0.25);
    xpoly(xx+ww*[0.2;0.5],yy+hh*[0.45;0.45],color=color(0,0,0),thickness=0.25);
    xpoly(xx+ww*[0.45;0.6],yy+hh*[0.35;0.35],color=color(0,0,0),thickness=0.25);
    xpoly(xx+ww*[0.1;0.2],yy+hh*[0.35;0.35],color=color(0,0,0),thickness=0.25);
    xpoly(xx+ww*[0.1;0.1],yy+hh*[0.66;0.35],color=color(0,0,0),thickness=0.25);
    xpoly(xx+ww*[0.6;0.6],yy+hh*[0.65;0.35],color=color(0,0,0),thickness=0.25);
    xpoly(xx+ww*[0.05;0.1],yy+hh*[0.5;0.5],color=color(0,0,0),thickness=0.25);
    xpoly(xx+ww*[0.95;0.9],yy+hh*[0.5;0.5],color=color(0,0,0),thickness=0.25);
    xpoly(xx+ww*[0.6;0.8;0.8],yy+hh*[0.5;0.5;0.35],color=color(0,0,0),thickness=0.25);
    xpoly(xx+ww*[0.7;0.7;0.9;0.9],yy+hh*[0.44;0.3;0.3;0.5],color=color(0,0,0),thickness=0.25);
    if orient then
      xstring(orig(1)+sz(1)*-0.25,orig(2)+sz(2)*-0.15,"b="+string(b)+"",w=sz(1)*1.5,h=sz(2)*0.2,fill=%t,color=color(0,0,0));
    else
      xstring(orig(1)+sz(1)*(-0.25),orig(2)+sz(2)*-0.15,"b="+string(b)+"",w=sz(1)*1.5,h=sz(2)*0.2,fill=%t,color=color(0,0,0));
    end
    if orient then
      xstring(orig(1)+sz(1)*-0.25,orig(2)+sz(2)*0.8,model.label,w=sz(1)*1.5,h=sz(2)*0.2,fill=%t,color=color(0,0,255));
    else
      xstring(orig(1)+sz(1)*(-0.25),orig(2)+sz(2)*0.8,model.label,w=sz(1)*1.5,h=sz(2)*0.2,fill=%t,color=color(0,0,255));
    end
    if orient then
      xstring(orig(1)+sz(1)*-0.26,orig(2)+sz(2)*0.04,"c="+string(c)+"",w=sz(1)*1.5,h=sz(2)*0.2,fill=%t,color=color(0,0,0));
    else
      xstring(orig(1)+sz(1)*(-0.24),orig(2)+sz(2)*0.04,"c="+string(c)+"",w=sz(1)*1.5,h=sz(2)*0.2,fill=%t,color=color(0,0,0));
    end
  endfunction

x=[];y=[];typ=[];

select job
  case 'plot' then
    c=arg1.graphics.exprs(1);
    d=arg1.graphics.exprs(2);
    b=arg1.graphics.exprs(3);
    phi_rel0=arg1.graphics.exprs(4);
    standard_draw(arg1,%f,_MMRI_TwoFlanges_dp);
  case 'getinputs' then
    [x,y,typ]=_MMRI_TwoFlanges_ip(arg1);
  case 'getoutputs' then
    [x,y,typ]=_MMRI_TwoFlanges_op(arg1);
  case 'getorigin' then
    [x,y]=standard_origin(arg1);
  case 'set' then
    x=arg1;
    graphics=arg1.graphics;exprs=graphics.exprs;
    model=arg1.model;
    while %t do
      [ok,c,d,b,phi_rel0,exprs]=..
        getvalue(['';'CMRC_ElastoBacklash';'';'Backlash connected in series to linear spring and damper (backlash is modeled with elasticity)';''],..
        [' c [N.m/rad] : Spring constant (c > 0 required)',' d [N.m.s/rad] : Damping constant (d >= 0)',' b [rad] : Total backlash (b >= 0)',' phi_rel0 [rad] : Unstretched spring angle'],..
        list('vec',1,'vec',1,'vec',1,'vec',1),exprs);
      if ~ok then break, end
      model.equations.parameters(2)=list(c,d,b,phi_rel0)
      model.in=[1];
      model.out=[1];
      graphics.exprs=exprs;
      x.graphics=graphics;x.model=model;
      break
    end
  case 'define' then
    c=100000;
    d=0;
    b=0;
    phi_rel0=0;
    model=scicos_model();
    model.sim='Coselica';
    model.blocktype='c';
    model.dep_ut=[%t %f];
    model.in=[1];
    model.out=[1];
    mo=modelica();
      mo.model='Coselica.Mechanics.Rotational.Components.ElastoBacklash';
      mo.inputs=['flange_a'];
      mo.outputs=['flange_b'];
      mo.parameters=list(['c','d','b','phi_rel0'],..
                         list(c,d,b,phi_rel0),..
                         [0,0,0,0]);
    model.equations=mo;
    exprs=[strcat(sci2exp(c));strcat(sci2exp(d));strcat(sci2exp(b));strcat(sci2exp(phi_rel0))];
    gr_i="blk_draw(sz,orig,orient,model.label)";
    x=standard_define([2 2],model,exprs,list(gr_i,0));
    x.graphics.in_implicit=['I'];
    x.graphics.out_implicit=['I'];
  end
endfunction
