// Coselica Toolbox for Scicos/Nsp
// Copyright (C) 2009-2016  Dirk Reusch, Kybernetik Dr. Reusch
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.

function [x,y,typ]=CMRC_Inertia(job,arg1,arg2)

  function blk_draw(sz,orig,orient,label)
    if orient then
      xx=orig(1);yy=orig(2);
      ww=sz(1);hh=sz(2);
    else
      xx=orig(1)+sz(1);yy=orig(2);
      ww=-sz(1);hh=sz(2);
    end
    if orient then
      xrect(orig(1)+sz(1)*0,orig(2)+sz(2)*0.55,sz(1)*0.25,sz(2)*0.1,color=color(0,0,0),background=color(192,192,192),thickness=0.25);
    else
      xrect(orig(1)+sz(1)*(1-0-0.25),orig(2)+sz(2)*0.55,sz(1)*0.25,sz(2)*0.1,color=color(0,0,0),background=color(192,192,192),thickness=0.25);
    end
    if orient then
      xrect(orig(1)+sz(1)*0.75,orig(2)+sz(2)*0.55,sz(1)*0.25,sz(2)*0.1,color=color(0,0,0),background=color(192,192,192),thickness=0.25);
    else
      xrect(orig(1)+sz(1)*(1-0.75-0.25),orig(2)+sz(2)*0.55,sz(1)*0.25,sz(2)*0.1,color=color(0,0,0),background=color(192,192,192),thickness=0.25);
    end
    xpoly(xx+ww*[0.1;0.2],yy+hh*[0.375;0.375],color=color(0,0,0),thickness=0.25);
    xpoly(xx+ww*[0.8;0.9],yy+hh*[0.375;0.375],color=color(0,0,0),thickness=0.25);
    xpoly(xx+ww*[0.15;0.15],yy+hh*[0.375;0.15],color=color(0,0,0),thickness=0.25);
    xpoly(xx+ww*[0.85;0.85],yy+hh*[0.375;0.15],color=color(0,0,0),thickness=0.25);
    xpoly(xx+ww*[0.1;0.2],yy+hh*[0.625;0.625],color=color(0,0,0),thickness=0.25);
    xpoly(xx+ww*[0.8;0.9],yy+hh*[0.625;0.625],color=color(0,0,0),thickness=0.25);
    xpoly(xx+ww*[0.15;0.15],yy+hh*[0.725;0.625],color=color(0,0,0),thickness=0.25);
    xpoly(xx+ww*[0.85;0.85],yy+hh*[0.725;0.625],color=color(0,0,0),thickness=0.25);
    xpoly(xx+ww*[0.15;0.85],yy+hh*[0.15;0.15],color=color(0,0,0),thickness=0.25);
    if orient then
      xrect(orig(1)+sz(1)*0.25,orig(2)+sz(2)*0.75,sz(1)*0.5,sz(2)*0.5,color=color(0,0,0),background=color(192,192,192),thickness=0.25);
    else
      xrect(orig(1)+sz(1)*(1-0.25-0.5),orig(2)+sz(2)*0.75,sz(1)*0.5,sz(2)*0.5,color=color(0,0,0),background=color(192,192,192),thickness=0.25);
    end
    if orient then
      xstring(orig(1)+sz(1)*-0.25,orig(2)+sz(2)*0.8,model.label,w=sz(1)*1.5,h=sz(2)*0.2,fill=%t,color=color(0,0,255));
    else
      xstring(orig(1)+sz(1)*(-0.25),orig(2)+sz(2)*0.8,model.label,w=sz(1)*1.5,h=sz(2)*0.2,fill=%t,color=color(0,0,255));
    end
    if orient then
      xstring(orig(1)+sz(1)*-0.25,orig(2)+sz(2)*-0.1,"J="+string(J)+"",w=sz(1)*1.5,h=sz(2)*0.2,fill=%t,color=color(0,0,0));
    else
      xstring(orig(1)+sz(1)*(-0.25),orig(2)+sz(2)*-0.1,"J="+string(J)+"",w=sz(1)*1.5,h=sz(2)*0.2,fill=%t,color=color(0,0,0));
    end
  endfunction

x=[];y=[];typ=[];

select job
  case 'plot' then
    J=arg1.graphics.exprs(1);
    initType=arg1.graphics.exprs(2);
    phi_start=arg1.graphics.exprs(3);
    w_start=arg1.graphics.exprs(4);
    standard_draw(arg1,%f,_MMRI_TwoFlanges_dp);
  case 'getinputs' then
    [x,y,typ]=_MMRI_TwoFlanges_ip(arg1);
  case 'getoutputs' then
    [x,y,typ]=_MMRI_TwoFlanges_op(arg1);
  case 'getorigin' then
    [x,y]=standard_origin(arg1);
  case 'set' then
    x=arg1;
    graphics=arg1.graphics;exprs=graphics.exprs;
    model=arg1.model;
    while %t do
      [ok,J,initType,phi_start,w_start,exprs]=..
        getvalue(['';'CMRC_Inertia';'';'1D-rotational component with inertia';''],..
        [' J [kg.m2] : Moment of inertia (>=0)',' initType [-] : Type of initial value for [phi,w] (0=guess,1=fixed)',' phi_start [rad] : Initial value for absolute rotation of component',' w_start [rad/s] : Initial value for absolute angular velocity of component'],..
        list('vec',1,'vec',2,'vec',1,'vec',1),exprs);
      if ~ok then break, end
      model.equations.parameters(2)=list(J,initType,phi_start,w_start)
      model.in=[1];
      model.out=[1];
      graphics.exprs=exprs;
      x.graphics=graphics;x.model=model;
      break
    end
  case 'define' then
    J=1;
    initType=[0,0];
    phi_start=0;
    w_start=0;
    model=scicos_model();
    model.sim='Coselica';
    model.blocktype='c';
    model.dep_ut=[%t %f];
    model.in=[1];
    model.out=[1];
    mo=modelica();
      mo.model='Coselica.Mechanics.Rotational.Components.Inertia';
      mo.inputs=['flange_a'];
      mo.outputs=['flange_b'];
      mo.parameters=list(['J','initType','phi_start','w_start'],..
                         list(J,initType,phi_start,w_start),..
                         [0,0,0,0]);
    model.equations=mo;
    exprs=[strcat(sci2exp(J));strcat(sci2exp(initType));strcat(sci2exp(phi_start));strcat(sci2exp(w_start))];
    gr_i="blk_draw(sz,orig,orient,model.label)";
    x=standard_define([2 2],model,exprs,list(gr_i,0));
    x.graphics.in_implicit=['I'];
    x.graphics.out_implicit=['I'];
  end
endfunction
