// Coselica Toolbox for Scicos/Nsp
// Copyright (C) 2009-2016  Dirk Reusch, Kybernetik Dr. Reusch
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.

function [x,y,typ]=CMRC_OneWayClutch(job,arg1,arg2)

  function blk_draw(sz,orig,orient,label)
    if orient then
      xx=orig(1);yy=orig(2);
      ww=sz(1);hh=sz(2);
    else
      xx=orig(1)+sz(1);yy=orig(2);
      ww=-sz(1);hh=sz(2);
    end
    xfpoly(xx+ww*[0.35;0.2;0.2;0.35],yy+hh*[0.7;0.75;0.65;0.7],color=color(0,0,127),fill_color=color(0,0,127),thickness=0.25);
    xpoly(xx+ww*[0.5;0.05;0.05;0.35],yy+hh*[0.95;0.85;0.7;0.7],color=color(0,0,127),thickness=0.25);
    xpoly(xx+ww*[0.5;0.95;0.95;0.65],yy+hh*[0.95;0.85;0.7;0.7],color=color(0,0,127),thickness=0.25);
    xpoly(xx+ww*[0.5;0.5],yy+hh*[0.95;1],color=color(0,0,127),thickness=0.25);
    xfpoly(xx+ww*[0.65;0.8;0.8;0.65],yy+hh*[0.7;0.75;0.65;0.7],color=color(0,0,127),fill_color=color(0,0,127),thickness=0.25);
    if orient then
      xrect(orig(1)+sz(1)*0.55,orig(2)+sz(2)*0.8,sz(1)*0.1,sz(2)*0.6,color=color(0,0,0),background=color(192,192,192),thickness=0.25);
    else
      xrect(orig(1)+sz(1)*(1-0.55-0.1),orig(2)+sz(2)*0.8,sz(1)*0.1,sz(2)*0.6,color=color(0,0,0),background=color(192,192,192),thickness=0.25);
    end
    if orient then
      xrect(orig(1)+sz(1)*0.35,orig(2)+sz(2)*0.8,sz(1)*0.1,sz(2)*0.6,color=color(0,0,0),background=color(192,192,192),thickness=0.25);
    else
      xrect(orig(1)+sz(1)*(1-0.35-0.1),orig(2)+sz(2)*0.8,sz(1)*0.1,sz(2)*0.6,color=color(0,0,0),background=color(192,192,192),thickness=0.25);
    end
    if orient then
      xrect(orig(1)+sz(1)*0.65,orig(2)+sz(2)*0.55,sz(1)*0.35,sz(2)*0.1,color=color(0,0,0),background=color(192,192,192),thickness=0.25);
    else
      xrect(orig(1)+sz(1)*(1-0.65-0.35),orig(2)+sz(2)*0.55,sz(1)*0.35,sz(2)*0.1,color=color(0,0,0),background=color(192,192,192),thickness=0.25);
    end
    if orient then
      xrect(orig(1)+sz(1)*0,orig(2)+sz(2)*0.55,sz(1)*0.35,sz(2)*0.1,color=color(0,0,0),background=color(192,192,192),thickness=0.25);
    else
      xrect(orig(1)+sz(1)*(1-0-0.35),orig(2)+sz(2)*0.55,sz(1)*0.35,sz(2)*0.1,color=color(0,0,0),background=color(192,192,192),thickness=0.25);
    end
    if orient then
      xstring(orig(1)+sz(1)*0,orig(2)+sz(2)*-0.1,model.label,w=sz(1)*1,h=sz(2)*0.2,fill=%t,color=color(0,0,0));
    else
      xstring(orig(1)+sz(1)*(0),orig(2)+sz(2)*-0.1,model.label,w=sz(1)*1,h=sz(2)*0.2,fill=%t,color=color(0,0,0));
    end
    xfpoly(xx+ww*[0.45;0.75;0.45;0.45],yy+hh*[0.65;0.5;0.35;0.65],color=color(0,0,0),fill_color=color(0,0,0),thickness=0.25);
  endfunction

x=[];y=[];typ=[];

select job
  case 'plot' then
    mue_pos=arg1.graphics.exprs(1);
    peak=arg1.graphics.exprs(2);
    cgeo=arg1.graphics.exprs(3);
    fn_max=arg1.graphics.exprs(4);
    standard_draw(arg1,%f,_CMR_OneWayClutch_dp);
  case 'getinputs' then
    [x,y,typ]=_CMR_OneWayClutch_ip(arg1);
  case 'getoutputs' then
    [x,y,typ]=_CMR_OneWayClutch_op(arg1);
  case 'getorigin' then
    [x,y]=standard_origin(arg1);
  case 'set' then
    x=arg1;
    graphics=arg1.graphics;exprs=graphics.exprs;
    model=arg1.model;
    while %t do
      [ok,mue_pos,peak,cgeo,fn_max,exprs]=..
        getvalue(['';'CMRC_OneWayClutch';'';'';''],..
        [' mue_pos [-] : positive sliding friction coefficient (w_rel>=0)',' peak [-] : peak >= 1, peak*mue_pos = maximum value of mue for w_rel==0',' cgeo [-] : cgeo >= 0, Geometry constant containing friction distribution assumption',' fn_max [N] : fn_max >= 0, Maximum normal force'],..
        list('vec',1,'vec',1,'vec',1,'vec',1),exprs);
      if ~ok then break, end
      model.equations.parameters(2)=list(mue_pos,peak,cgeo,fn_max)
      model.in=[1;1];
      model.out=[1];
      graphics.exprs=exprs;
      x.graphics=graphics;x.model=model;
      break
    end
  case 'define' then
    mue_pos=0.5;
    peak=1;
    cgeo=1;
    fn_max=1;
    model=scicos_model();
    model.sim='Coselica';
    model.blocktype='c';
    model.dep_ut=[%t %f];
    model.in=[1;1];
    model.out=[1];
    mo=modelica();
      mo.model='Coselica.Mechanics.Rotational.Components.OneWayClutch';
      mo.inputs=['flange_a','f_normalized'];
      mo.outputs=['flange_b'];
      mo.parameters=list(['mue_pos','peak','cgeo','fn_max'],..
                         list(mue_pos,peak,cgeo,fn_max),..
                         [0,0,0,0]);
    model.equations=mo;
    exprs=[strcat(sci2exp(mue_pos));strcat(sci2exp(peak));strcat(sci2exp(cgeo));strcat(sci2exp(fn_max))];
    gr_i="blk_draw(sz,orig,orient,model.label)";
    x=standard_define([2 2],model,exprs,list(gr_i,0));
    x.graphics.in_implicit=['I','I'];
    x.graphics.out_implicit=['I'];
  end
endfunction
