// Coselica Toolbox for Scicos/Nsp
// Copyright (C) 2009-2016  Dirk Reusch, Kybernetik Dr. Reusch
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.

function [x,y,typ]=CMTC_Pulley(job,arg1,arg2)

  function blk_draw(sz,orig,orient,label)
    if orient then
      xx=orig(1);yy=orig(2);
      ww=sz(1);hh=sz(2);
    else
      xx=orig(1)+sz(1);yy=orig(2);
      ww=-sz(1);hh=sz(2);
    end
    if orient then
      xarc(orig(1)+sz(1)*0.25,orig(2)+sz(2)*0.75,sz(1)*0.5,sz(2)*0.5,0,23040,color=color(0,0,0),background=color(255,255,255),thickness=0.25);
    else
      xarc(orig(1)+sz(1)*(1-0.25-0.5),orig(2)+sz(2)*0.75,sz(1)*0.5,sz(2)*0.5,-11520,23040,color=color(0,0,0),background=color(255,255,255),thickness=0.25);
    end
    xpoly(xx+ww*[0.05;0.45],yy+hh*[0.5;0.5],color=color(0,191,0),thickness=0.25);
    if orient then
      xarc(orig(1)+sz(1)*0.45,orig(2)+sz(2)*0.55,sz(1)*0.1,sz(2)*0.1,0,23040,color=color(0,0,0),background=color(0,0,0),thickness=0.25);
    else
      xarc(orig(1)+sz(1)*(1-0.45-0.1),orig(2)+sz(2)*0.55,sz(1)*0.1,sz(2)*0.1,-11520,23040,color=color(0,0,0),background=color(0,0,0),thickness=0.25);
    end
    xpoly(xx+ww*[0.5;0.95],yy+hh*[0.75;0.75],color=color(0,0,0),thickness=0.25);
    xpoly(xx+ww*[0.5;0.95],yy+hh*[0.25;0.25],color=color(0,0,0),thickness=0.25);
    if orient then
      xstring(orig(1)+sz(1)*0,orig(2)+sz(2)*0.85,model.label,w=sz(1)*1,h=sz(2)*0.2,fill=%t,color=color(0,0,255));
    else
      xstring(orig(1)+sz(1)*(0),orig(2)+sz(2)*0.85,model.label,w=sz(1)*1,h=sz(2)*0.2,fill=%t,color=color(0,0,255));
    end
    if orient then
      xstring(orig(1)+sz(1)*0.7,orig(2)+sz(2)*0.75,"1",w=sz(1)*0.175,h=sz(2)*0.15,fill=%t,color=color(0,0,0));
    else
      xstring(orig(1)+sz(1)*(0.125),orig(2)+sz(2)*0.75,"1",w=sz(1)*0.175,h=sz(2)*0.15,fill=%t,color=color(0,0,0));
    end
    if orient then
      xstring(orig(1)+sz(1)*0.7,orig(2)+sz(2)*0.1,"2",w=sz(1)*0.175,h=sz(2)*0.15,fill=%t,color=color(0,0,0));
    else
      xstring(orig(1)+sz(1)*(0.125),orig(2)+sz(2)*0.1,"2",w=sz(1)*0.175,h=sz(2)*0.15,fill=%t,color=color(0,0,0));
    end
  endfunction

x=[];y=[];typ=[];

select job
  case 'plot' then
    L=arg1.graphics.exprs(1);
    standard_draw(arg1,%f,_CMTI_PartialPulley_dp);
  case 'getinputs' then
    [x,y,typ]=_CMTI_PartialPulley_ip(arg1);
  case 'getoutputs' then
    [x,y,typ]=_CMTI_PartialPulley_op(arg1);
  case 'getorigin' then
    [x,y]=standard_origin(arg1);
  case 'set' then
    x=arg1;
    graphics=arg1.graphics;exprs=graphics.exprs;
    model=arg1.model;
    while %t do
      [ok,L,exprs]=..
        getvalue(['';'CMTC_Pulley';'';'Simple Pulley (without mass and inertia; geometric constraint)';''],..
        [' L [m] : Geometric constraint L=(s1+s2)/2-s'],..
        list('vec',1),exprs);
      if ~ok then break, end
      model.equations.parameters(2)=list(L)
      model.in=[1];
      model.out=[1;1];
      graphics.exprs=exprs;
      x.graphics=graphics;x.model=model;
      break
    end
  case 'define' then
    L=0;
    model=scicos_model();
    model.sim='Coselica';
    model.blocktype='c';
    model.dep_ut=[%t %f];
    model.in=[1];
    model.out=[1;1];
    mo=modelica();
      mo.model='Coselica.Mechanics.Translational.Components.Pulley';
      mo.inputs=['flange_a'];
      mo.outputs=['flange_b1','flange_b2'];
      mo.parameters=list(['L'],..
                         list(L),..
                         [0]);
    model.equations=mo;
    exprs=[strcat(sci2exp(L))];
    gr_i="blk_draw(sz,orig,orient,model.label)";
    x=standard_define([2 2],model,exprs,list(gr_i,0));
    x.graphics.in_implicit=['I'];
    x.graphics.out_implicit=['I','I'];
  end
endfunction
