// Coselica Toolbox for Scicos/Nsp
// Copyright (C) 2009-2016  Dirk Reusch, Kybernetik Dr. Reusch
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.

function [x,y,typ]=CTHC_ThermalConductor(job,arg1,arg2)

  function blk_draw(sz,orig,orient,label)
    if orient then
      xx=orig(1);yy=orig(2);
      ww=sz(1);hh=sz(2);
    else
      xx=orig(1)+sz(1);yy=orig(2);
      ww=-sz(1);hh=sz(2);
    end
    if orient then
      xrect(orig(1)+sz(1)*0.05,orig(2)+sz(2)*0.85,sz(1)*0.9,sz(2)*0.7,color=color(0,0,0),background=color(192,192,192),thickness=0.25);
    else
      xrect(orig(1)+sz(1)*(1-0.05-0.9),orig(2)+sz(2)*0.85,sz(1)*0.9,sz(2)*0.7,color=color(0,0,0),background=color(192,192,192),thickness=0.25);
    end
    xpoly(xx+ww*[0.05;0.05],yy+hh*[0.85;0.15],color=color(0,0,0),thickness=0.5);
    xpoly(xx+ww*[0.95;0.95],yy+hh*[0.85;0.15],color=color(0,0,0),thickness=0.5);
    if orient then
      xstring(orig(1)+sz(1)*-0.195,orig(2)+sz(2)*0.87,model.label,w=sz(1)*1.4,h=sz(2)*0.3,fill=%t,color=color(0,0,255));
    else
      xstring(orig(1)+sz(1)*(-0.205),orig(2)+sz(2)*0.87,model.label,w=sz(1)*1.4,h=sz(2)*0.3,fill=%t,color=color(0,0,255));
    end
    if orient then
      xstring(orig(1)+sz(1)*-0.075,orig(2)+sz(2)*-0.08,"G="+string(G)+"",w=sz(1)*1.14,h=sz(2)*0.2,fill=%t,color=color(0,0,0));
    else
      xstring(orig(1)+sz(1)*(-0.065),orig(2)+sz(2)*-0.08,"G="+string(G)+"",w=sz(1)*1.14,h=sz(2)*0.2,fill=%t,color=color(0,0,0));
    end
  endfunction

x=[];y=[];typ=[];

select job
  case 'plot' then
    G=arg1.graphics.exprs(1);
    standard_draw(arg1,%f,_MTHI_Element1D_dp);
  case 'getinputs' then
    [x,y,typ]=_MTHI_Element1D_ip(arg1);
  case 'getoutputs' then
    [x,y,typ]=_MTHI_Element1D_op(arg1);
  case 'getorigin' then
    [x,y]=standard_origin(arg1);
  case 'set' then
    x=arg1;
    graphics=arg1.graphics;exprs=graphics.exprs;
    model=arg1.model;
    while %t do
      [ok,G,exprs]=..
        getvalue(['';'CTHC_ThermalConductor';'';'Lumped thermal element transporting heat without storing it';''],..
        [' G [W/K] : Constant thermal conductance of material'],..
        list('vec',1),exprs);
      if ~ok then break, end
      model.equations.parameters(2)=list(G)
      model.in=[1];
      model.out=[1];
      graphics.exprs=exprs;
      x.graphics=graphics;x.model=model;
      break
    end
  case 'define' then
    G=1;
    model=scicos_model();
    model.sim='Coselica';
    model.blocktype='c';
    model.dep_ut=[%t %f];
    model.in=[1];
    model.out=[1];
    mo=modelica();
      mo.model='Coselica.Thermal.HeatTransfer.Components.ThermalConductor';
      mo.inputs=['port_a'];
      mo.outputs=['port_b'];
      mo.parameters=list(['G'],..
                         list(G),..
                         [0]);
    model.equations=mo;
    exprs=[strcat(sci2exp(G))];
    gr_i="blk_draw(sz,orig,orient,model.label)";
    x=standard_define([2 2],model,exprs,list(gr_i,0));
    x.graphics.in_implicit=['I'];
    x.graphics.out_implicit=['I'];
  end
endfunction
