// Coselica Toolbox for Scicos/Nsp
// Copyright (C) 2009-2016  Dirk Reusch, Kybernetik Dr. Reusch
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.

function [x,y,typ]=CTHS_FixedHeatFlow(job,arg1,arg2)

  function blk_draw(sz,orig,orient,label)
    if orient then
      xx=orig(1);yy=orig(2);
      ww=sz(1);hh=sz(2);
    else
      xx=orig(1)+sz(1);yy=orig(2);
      ww=-sz(1);hh=sz(2);
    end
    if orient then
      xstring(orig(1)+sz(1)*-0.17,orig(2)+sz(2)*0.8,model.label,w=sz(1)*1.33,h=sz(2)*0.3,fill=%t,color=color(0,0,255));
    else
      xstring(orig(1)+sz(1)*(-0.16),orig(2)+sz(2)*0.8,model.label,w=sz(1)*1.33,h=sz(2)*0.3,fill=%t,color=color(0,0,255));
    end
    if orient then
      xstring(orig(1)+sz(1)*-0.185,orig(2)+sz(2)*0,"Q_flow="+string(Q_flow)+"",w=sz(1)*1.35,h=sz(2)*0.24,fill=%t,color=color(0,0,0));
    else
      xstring(orig(1)+sz(1)*(-0.165),orig(2)+sz(2)*0,"Q_flow="+string(Q_flow)+"",w=sz(1)*1.35,h=sz(2)*0.24,fill=%t,color=color(0,0,0));
    end
    xpoly(xx+ww*[0;0.74],yy+hh*[0.4;0.4],color=color(191,0,0),thickness=0.5);
    xpoly(xx+ww*[0;0.73],yy+hh*[0.6;0.6],color=color(191,0,0),thickness=0.5);
    xfpoly(xx+ww*[0.7;0.7;0.85;0.7],yy+hh*[0.5;0.7;0.6;0.5],color=color(191,0,0),fill_color=color(191,0,0),thickness=0.25);
    xfpoly(xx+ww*[0.7;0.7;0.85;0.7],yy+hh*[0.3;0.5;0.4;0.3],color=color(191,0,0),fill_color=color(191,0,0),thickness=0.25);
    if orient then
      xrect(orig(1)+sz(1)*0.85,orig(2)+sz(2)*0.7,sz(1)*0.1,sz(2)*0.4,color=color(191,0,0),background=color(191,0,0),thickness=0.25);
    else
      xrect(orig(1)+sz(1)*(1-0.85-0.1),orig(2)+sz(2)*0.7,sz(1)*0.1,sz(2)*0.4,color=color(191,0,0),background=color(191,0,0),thickness=0.25);
    end
  endfunction

x=[];y=[];typ=[];

select job
  case 'plot' then
    Q_flow=arg1.graphics.exprs(1);
    T_ref=arg1.graphics.exprs(2);
    alpha=arg1.graphics.exprs(3);
    standard_draw(arg1,%f,_CTHS_FixedHeatFlow_dp);
  case 'getinputs' then
    [x,y,typ]=_CTHS_FixedHeatFlow_ip(arg1);
  case 'getoutputs' then
    [x,y,typ]=_CTHS_FixedHeatFlow_op(arg1);
  case 'getorigin' then
    [x,y]=standard_origin(arg1);
  case 'set' then
    x=arg1;
    graphics=arg1.graphics;exprs=graphics.exprs;
    model=arg1.model;
    while %t do
      [ok,Q_flow,T_ref,alpha,exprs]=..
        getvalue(['';'CTHS_FixedHeatFlow';'';'Fixed heat flow boundary condition';''],..
        [' Q_flow [W] : Fixed heat flow rate at port',' T_ref [K] : Reference temperature',' alpha [1/K] : Temperature coefficient of heat flow rate'],..
        list('vec',1,'vec',1,'vec',1),exprs);
      if ~ok then break, end
      model.equations.parameters(2)=list(Q_flow,T_ref,alpha)
      model.in=[];
      model.out=[1];
      graphics.exprs=exprs;
      x.graphics=graphics;x.model=model;
      break
    end
  case 'define' then
    Q_flow=1;
    T_ref=293.15;
    alpha=0;
    model=scicos_model();
    model.sim='Coselica';
    model.blocktype='c';
    model.dep_ut=[%t %f];
    model.in=[];
    model.out=[1];
    mo=modelica();
      mo.model='Coselica.Thermal.HeatTransfer.Sources.FixedHeatFlow';
      mo.inputs=[];
      mo.outputs=['port'];
      mo.parameters=list(['Q_flow','T_ref','alpha'],..
                         list(Q_flow,T_ref,alpha),..
                         [0,0,0]);
    model.equations=mo;
    exprs=[strcat(sci2exp(Q_flow));strcat(sci2exp(T_ref));strcat(sci2exp(alpha))];
    gr_i="blk_draw(sz,orig,orient,model.label)";
    x=standard_define([2 2],model,exprs,list(gr_i,0));
    x.graphics.in_implicit=[];
    x.graphics.out_implicit=['I'];
  end
endfunction
